// audio\music.aks
// export as music.bin from arkos, at address 1000 (leave "export as sfx only UNTICKED")
// this creates audio\music.bin
// run audio\audio.bat to create audio\music.asm

// edit music.asm to look like this:

/*
lzx0

defb #41,#54,#31,#30,#01,#40,#42,#0F,#02,#06,#21,#00,#10,#10,#19,#10
defb #00,#00,#00,#00,#00,#00,#0D,#12,#10,#01,#00,#3C,#38,#34,#30,#2C
...
defb #7B,#02,#81,#02,#85,#02,#89,#02,#7B,#02,#81,#02,#85,#02,#89,#00
defb #42,#80,#00,#00,#00

lzclose
*/

// copy music.asm to rasm\music.asm
// run rasm\_music.bat to create src\music.h
// in src\music.h, comment out the line starting with:
// const long int music_bin_size

// ****************************************************************

// check all rules

// for submission, folder structure in
// C:\Users\ErvinPajor\Downloads\Goatfish_ErvinPajor.zip

// cdt file!

// ****************************************************************

#include <cpctelera.h>
#include <stdio.h>
#include "prototype.h"
#include "define.h"
#include "data.h"
#include "player.h"
#include "sprite.h"
#include "font.h"
#include "music.h"
#include "level1_lz.h"
#include "level2_lz.h"
#include "level3_lz.h"

u8* const logoChar[]={
mainMenu_g_bin,mainMenu_o_bin,mainMenu_a_bin,mainMenu_t_bin,
mainMenu_f_bin,mainMenu_i_bin,mainMenu_s_bin,mainMenu_h_bin,
mainMenu_2_bin,
};

typedef struct{
    u8* image;
    u16 imageOffset;
    u8 imageWidth;
} typeTextImage;

typeTextImage const tblTextImage[]={
{mainMenu_underwater_bin,64*4+15,20},
{mainMenu_noOne_bin,64*4+37,12},
{mainMenu_canHear_bin,64*5+14,16},
{mainMenu_youBleat_bin,64*5+32,18},
};

u8* image;

void main(){
    cpct_disableFirmware();
    cpct_setStackLocation((u8*)0x8000);

    relocateData();
    cpct_waitVSYNC();
    cpct_setPalette(hw_palette_black,16);
    cpct_setBorder(HW_BLACK);
    cpct_setVideoMode(0);
    shrinkScreen();
    decrunchData();
    populateYaddress();

    highScore=0;
    
    gameState=STATE_MAIN_MENU;
    textPage();
    prepareMusic();

    while(1){
        if (gameState==STATE_MAIN_MENU){
            if (textDone==0){
                logoX=3;

                for (u8 i=0;i<9;i++){
                    if (i==8){
                        sleepLogo();
                        logoX+=5;
                    }

                    image=logoChar[i];

                    __asm
                    ld hl,(_image)
                    __endasm;

                    drawLogo();
                }

                sleepLogo();
                sleepLogo();

                for (u8 i=0;i<4;i++){
                    image=tblTextImage[i].image;

                    __asm
                    ld hl,(_image)
                    __endasm;

                    decrunchFont();
                    cpct_drawSprite(spriteScratch,CPCT_VMEM_START+tblTextImage[i].imageOffset,tblTextImage[i].imageWidth,8);
                }

                print_mainMenu();
                textDone=1;
            }

            if (musicPlay==1){
                cpct_akp_musicPlay();
            }

            musicPlay++;

            if (musicPlay>34){
                musicPlay=1;
            }

            waitForKey();

            if (cpct_isKeyPressed(Key_I)){
                gameState=STATE_INSTRUCTIONS;
                textPage();
            }
            else if (cpct_isKeyPressed(Key_C)){
                gameState=STATE_CREDITS;
                textPage();
            }
            else if (cpct_isAnyKeyPressed()){
                waitForKey();
                initGame();
                initLevel();
            }
        }
        else if (gameState==STATE_GAME_PLAY){
            processGameInput();

            if ((gameState==STATE_GAME_PLAY) || (gameState==STATE_GAME_COLLISION)){
                buildScreen();
                updateEnergy();
                updateFoodDisplay();
            }
        }
        else if (gameState==STATE_GAME_COLLISION){
            if (collide_behind_z!=0){
                silence();
                cpct_akp_SFXPlay(13,15,40,0,0,AY_CHANNEL_A);
                cpct_akp_SFXPlay(13,15,48,0,0,AY_CHANNEL_C);
            }

            sfxFoodPlayed=0;
            sfxAngryFacePlayed=0;
            sfxUrchinPlayed=0;
            sfxEolPlayed=0;

            processCollisions();

            if (sfxUrchinPlayed==1){
                sfxDeath(10);
                playerDead(16);
            }
            else if (sfxAngryFacePlayed==1){
                sfxDeath(11);
                playerDead(8);
            }
            else if (sfxEolPlayed==1){
                gameState=STATE_GAME_LEVEL_COMPLETE;
            }
        }        
        else if (gameState==STATE_GAME_PAUSED){
            pauseGame();
        }
        else if (gameState==STATE_GAME_OVER){
            if (textDone==0){
                print_gameOver();
                textDone=1;
            }

            waitForKey();

            if (cpct_isAnyKeyPressed()){
                gameState=STATE_MAIN_MENU;
                textPage();
                prepareMusic();
            }
        }
        else if (gameState==STATE_INSTRUCTIONS){
            if (textDone==0){
                print_instructions();
                textDone=1;
            }

            waitForKey();

            if (cpct_isAnyKeyPressed()){
                gameState=STATE_MAIN_MENU;
                textPage();
                prepareMusic();
            }
        }
        else if (gameState==STATE_CREDITS){
            if (textDone==0){
                print_credits();
                textDone=1;
            }

            waitForKey();

            if (cpct_isAnyKeyPressed()){
                gameState=STATE_MAIN_MENU;
                textPage();
                prepareMusic();
            }
        }
        else if (gameState==STATE_GAME_LEVEL_COMPLETE){
            __asm
            ld hl,#_fonts_wellDone_bin
            __endasm;

            pauseGamePrint();

            for (u8 i=0;i<128;i++){
                cpct_waitVSYNC();
                cpct_akp_musicPlay();
            }

            gameState=STATE_NEXT_LEVEL;
            textPage();
        }
        else if (gameState==STATE_NEXT_LEVEL){
            if (currentLevel<2){
                currentLevel++;

                for (u8 i=0;i<64;i++){
                    print_nextLevel();
                }

                initLevel();
            }
            else{
                for (u8 i=0;i<80;i++){
                    print_congratulations();
                }

                gameState=STATE_GAME_OVER;
                textPage();
            }
        }
        else if (gameState==STATE_DEAD){
            __asm
            ld hl,#_fonts_blueBar_bin
            call _printFood
            __endasm;

            sfxDeath(10);
            playerDead(16);
            textPage();

            for (u8 i=0;i<80;i++){
                print_dead();
            }

            gameState=STATE_GAME_OVER;
            textPage();
        }
    }
}

void prepareMusic(){
    __asm
    ld hl,#_music_bin
    ld de,#0x1000
    call _LZX0_decrunch
    __endasm;

    silence();
    cpct_akp_musicInit(LEVEL_BUFFER);
    cpct_akp_SFXInit(LEVEL_BUFFER);
    musicPlay=1;
}

void playerDead(u8 deathReps){
    player_dead=1;

    for (u8 i=0;i<deathReps;i++){
        buildScreen();

        if (player_dead==1){
            player_dead=2;
        }
        else{
            player_dead=1;
        }
    }

    player_dead=0;
}

void sleepLogo(){
__asm

ld c,#96

LOOP_SLEEP_C:
ld b,#255

LOOP_SLEEP_B:
djnz LOOP_SLEEP_B

dec c
jr nz,LOOP_SLEEP_C

__endasm;
}

void drawLogoSprite(u8 reps,u8 offset){
    u8* pSpriteScratch;
    u8* pVmem;

    for (u8 j=0;j<2;j++){
        cpct_waitVSYNC();
        pSpriteScratch=spriteScratch;
        pVmem=CPCT_VMEM_START+logoX+offset;

        for (u8 i=0;i<reps;i++){
            cpct_drawSprite(pSpriteScratch,pVmem,5,8);
            pSpriteScratch+=40;
            pVmem+=64;
        }
    }
}

void drawLogo(){
    decrunchFont();
    drawLogoSprite(1,128);
    drawLogoSprite(2,64);
    drawLogoSprite(3,0);
    logoX+=6;
}

void initGame(){
    silence();
    cpct_akp_musicInit(sfx_bin);
    cpct_akp_SFXInit(sfx_bin);

    score=0;
    currentLevel=0;
}

void initLevel(){
    decrunchLevel();

    gameState=STATE_GAME_PLAY;
    silence();

    cpct_waitVSYNC();
    cpct_setPalette(hw_palette_black,16);

    vmem=CPCT_VMEM_START;
    cls24(CLS_BLUE);

    vmem=SCREEN_BUFFER;
    cls24(CLS_BLUE);

    page=0xC0;
    yAddr=yAddr80;
    prepareTile_setPage();

    player_x=28;
    player_y=32;
    player_z=1536;

    player_maxSpeed=MAX_MOVE_SPEED;
    player_moveSpeed=MAX_MOVE_SPEED;
    player_direction=PLAYER_DIRECTION_RIGHT;
    
    player_frame=0;
    player_dead=0;

    energy=100;
    energyDecrement=1;
    energyDecrementDelay=ENERGY_DECREMENT_DELAY-currentLevel;

    firstBuildScreen=1;
    clearCollisions();

    __asm
    ld hl,#_fonts_goatfish2_bin
    call _decrunchFont
    __endasm;

    printWordSprite(CPCT_VMEM_START+64*20+22,20,8);

    __asm
    ld hl,#_fonts_score_bin
    call _decrunchFont
    __endasm;

    printWordSprite(CPCT_VMEM_START+64*22+12,10,8);
    printScore();

    __asm
    ld hl,#_fonts_energy_bin
    call _decrunchFont
    __endasm;

    printWordSprite(CPCT_VMEM_START+64*22+26,12,8);
    printEnergy();

    __asm
    ld hl,#_fonts_high_bin
    call _decrunchFont
    __endasm;

    printWordSprite(CPCT_VMEM_START+64*22+43,8,8);
    printHighScore();

    foodDisplay=FOOD_DISPLAY;

    __asm
    ld hl,#_fonts_letsgoat_bin
    jp _printFood
    __endasm;
}

void pauseGamePrint(){
    decrunchFont();
    printWordSprite(CPCT_VMEM_START+64*21+21,22,8);
}

void pauseGame(){
    foodDisplay=0;

    __asm
    ld hl,#_fonts_paused_bin
    __endasm;

    pauseGamePrint();
    waitForKey();

    if (cpct_isAnyKeyPressed()){
        gameState=STATE_GAME_PLAY;
        waitForKey();

        __asm
        ld hl,#_fonts_blueBar_bin
        jp _printFood
        __endasm;
    }
}

void decrunchFont(){
__asm
ld de,#_spriteScratch
jp _LZX0_decrunch
__endasm;
}

void silence(){
    cpct_akp_stop();
    cpct_akp_SFXStopAll();
}

void buildScreen(){
    // cls20();
    // prepareUrchins();
    // drawRects();
    // drawSprites();
    // drawPlayer();

    // if (collide_behind_z!=0){
    //     ptrZone=level[collide_behind_zoneIndex].zone;

    //     if (ptrZone!=0){
    //         while (ptrZone->sprite!=0){
    //             if (ptrZone->z==collide_behind_z){
    //                 player_z--;
    //                 processSprite();
    //                 player_z++;
    //             }

    //             ptrZone++;
    //         }
    //     }
    // }

    // drawScreen();

    // if (toPrintScore!=0){
    //     printScore();
    //     toPrintScore--;
    // }

    // changeVideoMemoryPage();

    // if (firstBuildScreen==1){
    //     cpct_waitVSYNC();
    //     cpct_setPalette(hw_palette_colour,16);
    //     cpct_setBorder(HW_BLUE);
    //     firstBuildScreen=0;
    // }

__asm

;src/main.c:156: cls20();
    call    _cls20
    call cpct_akp_musicPlay_asm
;src/main.c:157: prepareUrchins();
    call    _prepareUrchins
    call cpct_akp_musicPlay_asm
;src/main.c:158: drawRects();
    call    _drawRects
    call cpct_akp_musicPlay_asm
;src/main.c:159: drawSprites();
    call    _drawSprites
    call cpct_akp_musicPlay_asm
;src/main.c:160: drawPlayer();
    call    _drawPlayer
    call cpct_akp_musicPlay_asm
;src/main.c:162: if (collide_behind_z!=0){
    ld  hl, #_collide_behind_z
    inc hl
    ld a,(hl)
    dec hl
    or a,(hl)
    jr  Z,00109$
;src/main.c:163: ptrZone=level[collide_behind_zoneIndex].zone;
    ld  bc, #_level+0
    ld  hl, (_collide_behind_zoneIndex)
    ld  h, #0x00
    add hl, hl
    add hl, bc
    ld  a, (hl)
    ld  de, #_ptrZone
    ld  (de), a
    inc hl
    ld  a, (hl)
    inc de
    ld (de),a
;src/main.c:165: if (ptrZone!=0){
    dec de
    ex de,hl
    or  a,(hl)
    jr  Z,00109$
;src/main.c:166: while (ptrZone->sprite!=0){
00103$:
    ld  hl, (_ptrZone)
    push    hl
    ld  c, (hl)
    inc hl
    ld  b, (hl)
    pop hl
    ld  a, b
    or  a,c
    jr  Z,00109$
;src/main.c:167: if (ptrZone->z==collide_behind_z){
    inc hl
    inc hl
    ld  c, (hl)
    inc hl
    ld  b, (hl)
    ld  de, #_collide_behind_z
    ld  a, (de)
    sub a, c
    jr  NZ,00102$
    inc de
    ld  a, (de)
    sub a, b
    jr  NZ,00102$
;src/main.c:168: player_z--;
    ld  hl, (_player_z)
    dec hl
    ld  (_player_z), hl
;src/main.c:169: processSprite();
    call    _processSprite
;src/main.c:170: player_z++;
    ld  hl, #_player_z
    inc (hl)
    jr  NZ,00142$
    inc hl
    inc (hl)
00142$:
00102$:
;src/main.c:173: ptrZone++;
    ld  hl, #_ptrZone
    ld  a, (hl)
    add a, #0x04
    ld  (hl), a
    inc hl
    ld  a, (hl)
    adc a, #0x00
    ld  (hl), a
    jr  00103$
00109$:
;src/main.c:178: drawScreen();
    call    _drawScreen
    call cpct_akp_musicPlay_asm
// ;src/main.c:179: printDiag();
//     call    _printDiag
// ;src/main.c:181: if (toPrintScore!=0){
//     ld  a,(#_toPrintScore + 0)
//     or  a, a
//     jr  Z,00111$
// ;src/main.c:182: printScore();
//     call    _printScore
// ;src/main.c:183: toPrintScore--;
//     ld  hl, #_toPrintScore+0
//     dec (hl)
// 00111$:
;src/main.c:186: changeVideoMemoryPage();
    call    _changeVideoMemoryPage
;src/main.c:188: if (firstBuildScreen==1){
    ld  a,(#_firstBuildScreen + 0)
    dec a
    ret NZ
;src/main.c:189: cpct_waitVSYNC();
    call    _cpct_waitVSYNC
;src/main.c:190: cpct_setPalette(hw_palette_colour,16);
    ld  hl, #0x0010
    push    hl
    ld  hl, #_hw_palette_colour
    push    hl
    call    _cpct_setPalette
;src/main.c:191: cpct_setBorder(HW_BLUE);
    ld  hl, #0x0410
    push    hl
    call    _cpct_setPALColour
;src/main.c:192: firstBuildScreen=0;
    ld  hl,#_firstBuildScreen + 0
    ld  (hl), #0x00

__endasm;
}

void processCollisions(){
    // u8 screenBuilt=0;

    // if (collide_behind_z==0){
    //     for (u8 i=0;i<5;i++){
    //         if (tblCollision[i].tileID==255){
    //             break;
    //         }

    //         if (tblCollision[i].tileID>=16){
    //             tileByte=tblCollision[i].tileByte;
    //             ptrSprite=tblCollision[i].ptrSprite;
    //             removeFood();
    //         }

    //         tblCollisionFunction[tblCollision[i].tileID]();

    //         // if bit 5 is set (delay) or if the tile is food (tileID>=16)

    //         if (((tblCollision[i].tileByte & 0b00100000)!=0) || (tblCollision[i].tileID>=16)){
    //             buildScreen();
    //             buildScreen();
    //             screenBuilt=1;
    //         }
    //     }
    // }
    // else{
    //     buildScreen();
    // }

    // if (screenBuilt==0){
    //     buildScreen();
    // }

    // gameState=STATE_GAME_PLAY;

__asm

// ;src/main.c:197: u8 screenBuilt=0;
//     xor a
//     ld (_screenBuilt),a
;src/main.c:199: if (collide_behind_z==0){
    ld  hl, #_collide_behind_z
    inc hl
    ld  a, (hl)
    dec hl
    or  a,(hl)
    jr  NZ, 00110$
;src/main.c:200: for (u8 i=0;i<5;i++){
    ld  de, #_tblCollisionFunction+0
    xor a
    ld (_i),a
00115$:
    ld a,(_i)
    sub a, #0x05
    jr  NC, 00111$
;src/main.c:201: if (tblCollision[i].tileID==255){
    ld  bc, #_tblCollision+0
    ld hl,(_i)
    ld  h, #0x00
    add hl, hl
    add hl, hl
    add hl,bc
    ld (_collision_tileID),hl
    ld  l, (hl)
    ld  a, l
    inc a
    jr  Z,00111$
;src/main.c:206: tileByte=tblCollision[i].tileByte;
    ld bc,(_collision_tileID)
    inc bc
;src/main.c:205: if (tblCollision[i].tileID>=16){
    ld  a, l
    sub a, #0x10
    jr  C,00104$
;src/main.c:206: tileByte=tblCollision[i].tileByte;
    ld  a, (bc)
    ld  (#_tileByte + 0),a
;src/main.c:207: ptrSprite=tblCollision[i].ptrSprite;
    ld hl,(_collision_tileID)
    inc hl
    inc hl
    ld  a, (hl)
    push    bc
    push    de
    ld  de, #_ptrSprite
    ld  (de), a
    inc hl
    ld  a, (hl)
    inc de
    ld  (de), a
;src/main.c:208: removeFood();
    call    _removeFood
    pop de
    pop bc
00104$:
;src/main.c:211: tblCollisionFunction[tblCollision[i].tileID]();
    ld hl,(_collision_tileID)
    ld  l, (hl)
    ld  h, #0x00
    add hl, hl
    add hl, de
    ld  a, (hl)
    inc hl
    ld  h, (hl)
    ld  l, a
    push    bc
    push    de
    call    ___sdcc_call_hl
    pop de
    pop bc
// ;src/main.c:215: if (((tblCollision[i].tileByte & 0b00100000)!=0) || (tblCollision[i].tileID>=16)){
//     ld  a, (bc)
//     bit 5, a
//     jr  NZ,00105$
//     ld hl,(_collision_tileID)
//     ld  a, (hl)
//     sub a, #0x10
//     jr  C,00116$
// 00105$:
// ;src/main.c:216: buildScreen();
    // push    de
    // call    _buildScreen
    // call    _buildScreen
    // pop de
// ;src/main.c:218: screenBuilt=1;
    // ld hl,#_screenBuilt
    // ld (hl),#1
// 00116$:
;src/main.c:200: for (u8 i=0;i<5;i++){
    ld hl,#_i
    inc (hl)
    jr  00115$
00110$:
;src/main.c:223: buildScreen();
    call    _buildScreen
00111$:
// ;src/main.c:226: if (screenBuilt==0){
//     ld a,(_screenBuilt)
//     or  a, a
//     jr  NZ,00113$
// ;src/main.c:227: buildScreen();
//     call    _buildScreen
// 00113$:
;src/main.c:230: gameState=STATE_GAME_PLAY;
    ld  hl,#_gameState + 0
    ld  (hl), #0x01

__endasm;
}

void waitForKey(){
    while(cpct_isAnyKeyPressed()){
        silence();
        cpct_scanKeyboard_f();
    }

    cpct_scanKeyboard_f();
}

void processGameInput(){
    cpct_scanKeyboard_f();

    if (cpct_isKeyPressed(Key_P)){
        gameState=STATE_GAME_PAUSED;
        silence();
        return;
    }
    
    if (cpct_isKeyPressed(Key_Esc)){
        gameState=STATE_GAME_OVER;
        textPage();
        return;
    }

    if (player_y!=0){
        if (cpct_isKeyPressed(Key_CursorUp) || cpct_isKeyPressed(Joy0_Up)){
            player_y-=2;
        }
    }

    if (player_y!=64){
        if (cpct_isKeyPressed(Key_CursorDown) || cpct_isKeyPressed(Joy0_Down)){
            player_y+=2;
        }
    }

    if (player_x!=0){
        if (cpct_isKeyPressed(Key_CursorLeft) || cpct_isKeyPressed(Joy0_Left)){
            player_x--;
            player_direction=PLAYER_DIRECTION_LEFT;
        }
    }

    if (player_x!=56){
        if (cpct_isKeyPressed(Key_CursorRight) || cpct_isKeyPressed(Joy0_Right)){
            player_x++;
            player_direction=PLAYER_DIRECTION_RIGHT;
        }
    }

    if (cpct_isKeyPressed(Key_Space) || cpct_isKeyPressed(Joy0_Fire1)){
        player_maxSpeed=MAX_DIVE_SPEED;
        energyDecrement=2;
    }
    else{
        player_maxSpeed=MAX_MOVE_SPEED;
        energyDecrement=1;
    }

    // if (cpct_isKeyPressed(Key_Z) || cpct_isKeyPressed(Joy0_Fire1)){
        analyseCollisions();

        if (gameState==STATE_GAME_PLAY){
            player_z+=player_moveSpeed;
        }
    // }
}

void removeFood(){
    // // apple

    // if (tileByte==0b01010000){
    //     *ptrSprite=0b01000101;
    // }

    // // banana/hamburger

    // else if ((tileByte==0b01010001) || (tileByte==0b01010110)){
    //     *ptrSprite=0b01001101;
    // }


    // // cupcake/egg

    // else if ((tileByte==0b01010010) || (tileByte==0b01010011)){
    //     *ptrSprite=0b01001110;
    // }

    // // milkshake

    // else if (tileByte==0b01010100){
    //     *ptrSprite=0b01000110;
    // }

    // // pizza/drumstick

    // else if ((tileByte==0b01010111) || (tileByte==0b01010101)){
    //     *ptrSprite=0b01000111;
    // }

__asm

    ld  bc, #_tileByte
;src/main.c:696: *ptrSprite=0b01000101;
    ld  hl, (_ptrSprite)
;src/main.c:695: if (tileByte==0b01010000){
    ld a,(bc)
    cp a, #0x50
    jr  NZ,RF_00116
;src/main.c:696: *ptrSprite=0b01000101;
    ld  (hl), #0x45
    ret
RF_00116:
;src/main.c:701: else if ((tileByte==0b01010001) || (tileByte==0b01010110)){
    cp a, #0x51
    jr  Z,RF_00111
    cp a, #0x56
    jr  NZ,RF_00112
RF_00111:
;src/main.c:702: *ptrSprite=0b01001101;
    ld  (hl), #0x4d
    ret
RF_00112:
;src/main.c:708: else if ((tileByte==0b01010010) || (tileByte==0b01010011)){
    cp a, #0x52
    jr  Z,RF_00107
    cp a, #0x53
    jr  NZ,RF_00108
RF_00107:
;src/main.c:709: *ptrSprite=0b01001110;
    ld  (hl), #0x4e
    ret
RF_00108:
;src/main.c:714: else if (tileByte==0b01010100){
    cp a, #0x54
    jr  NZ,RF_00105
;src/main.c:715: *ptrSprite=0b01000110;
    ld  (hl), #0x46
    ret
RF_00105:
;src/main.c:720: else if ((tileByte==0b01010111) || (tileByte==0b01010101)){
    cp a, #0x57
    jr  Z,RF_00101
    cp a, #0x55
    ret NZ
RF_00101:
;src/main.c:721: *ptrSprite=0b01000111;
    ld  (hl), #0x47

__endasm;
}

void clearCollisions(){
    for (u8 i=0;i<5;i++){
        tblCollision[i].tileID=255; // 255 means "no tile"
        // tblCollision[i].tileByte=0;
    }

    collide_behind_z=0;
}

void checkTileCollisions(){
    // ptrSprite=ptrZone->sprite;

    // for (offsetY=0;offsetY<=4;offsetY++){
    //     collide_yFrom=offsetY*16+2;
    //     collide_yTo=collide_yFrom+15-4;

    //     for (offsetX=0;offsetX<=7;offsetX++){
    //         tileByte=*ptrSprite;

    //         if ((tileByte & 0b10000000)!=0){ // check bit 7
    //             ptrSprite++;
    //             break;
    //         }

    //         tileID=255;

    //         if ((tileByte & 0b00010000)!=0){ // check bit 4
    //             tileID=(tileByte & 0b00001111)+16;
    //         }
    //         else if ((tileByte & 0b01000000)!=0){ // check bit 6
    //             tileID=tileByte & 0b00001111;
    //         }

    //         if (tileID!=255){
    //             if (player_y<=collide_yTo){
    //                 if (player_y+15>=collide_yFrom){
    //                     collide_xFrom=offsetX*8+1;
    //                     collide_xTo=collide_xFrom+7-2;

    //                     if (player_x<=collide_xTo){
    //                         if (player_x+7>=collide_xFrom){
    //                             if (player_moveSpeed>0){
    //                                 for (u8 i=0;i<5;i++){
    //                                     if (tileID>=TILE_BIG_URCHIN_BEGIN && tileID<=TILE_BIG_URCHIN_END){
    //                                         tileID=TILE_URCHIN;
    //                                     }

    //                                     if (tileID==TILE_URCHIN && tblCollision[i].tileID==TILE_URCHIN){
    //                                         break;
    //                                     }

    //                                     if (tblCollision[i].tileID==255){
    //                                         gameState=STATE_GAME_COLLISION;
    //                                         tblCollision[i].tileID=tileID;
    //                                         tblCollision[i].tileByte=tileByte;
    //                                         tblCollision[i].ptrSprite=ptrSprite;
    //                                         break;
    //                                     }
    //                                 }
    //                             }
    //                             else{
    //                                 gameState=STATE_GAME_COLLISION;
    //                                 return;
    //                             }
    //                         }
    //                     }
    //                 }
    //             }
    //         }

    //         ptrSprite++;
    //     }
    // }

__asm

;src/main.c:310: ptrSprite=ptrZone->sprite;
    ld  hl, (_ptrZone)
    ld  a, (hl)
    ld  de, #_ptrSprite
    ld  (de), a
    inc hl
    ld  a, (hl)
    inc de
    ld  (de), a
;src/main.c:312: for (offsetY=0;offsetY<=4;offsetY++){
    ld  hl,#_offsetY + 0
    ld  (hl), #0x00
    ld  bc, #_tblCollision+0
00137$:
;src/main.c:313: collide_yFrom=offsetY*16+2;
    ld  a,(#_offsetY + 0)
    rlca
    rlca
    rlca
    rlca
    and a, #0xf0
    ld  hl, #_collide_yFrom
    add a, #0x02
    ld  (hl), a
;src/main.c:314: collide_yTo=collide_yFrom+15-4;
    ld  hl, #_collide_yTo
    ld  a,(#_collide_yFrom + 0)
    add a, #0x0b
    ld  (hl), a
;src/main.c:316: for (offsetX=0;offsetX<=7;offsetX++){
    ld  hl,#_offsetX + 0
    ld  (hl), #0x00
00135$:
;src/main.c:317: tileByte=*ptrSprite;
    ld  hl, (_ptrSprite)
    ld  a, (hl)
    ld  hl, #_tileByte
    ld  (hl), a
;src/main.c:319: if ((tileByte & 0b10000000)!=0){ // check bit 7
    bit 7, (hl)
    jr  Z,00102$
;src/main.c:320: ptrSprite++;
    ld  hl, #_ptrSprite
    inc (hl)
    jp  NZ,00138$
    inc hl
    inc (hl)
;src/main.c:321: break;
    jp  00138$
00102$:
;src/main.c:324: tileID=255;
    ld  hl,#_tileID + 0
    ld  (hl), #0xff
;src/main.c:327: tileID=(tileByte & 0b00001111)+16;
    ld  hl, #_tileByte
    ld  a, (hl)
    and a, #0x0f
;src/main.c:326: if ((tileByte & 0b00010000)!=0){ // check bit 4
    bit 4, (hl)
    jr  Z,00106$
;src/main.c:327: tileID=(tileByte & 0b00001111)+16;
    ld  hl, #_tileID
    add a, #0x10
    ld  (hl), a
    jr  00107$
00106$:
;src/main.c:329: else if ((tileByte & 0b01000000)!=0){ // check bit 6
    ld  hl,#_tileByte+0
    bit 6, (hl)
    jr  Z,00107$
;src/main.c:330: tileID=tileByte & 0b00001111;
    ld  (#_tileID + 0),a
00107$:
;src/main.c:333: if (tileID!=255){
    ld  a,(#_tileID + 0)
    inc a
    jp  Z,00129$
;src/main.c:334: if (player_y<=collide_yTo){
    ld  a, (#_collide_yTo)
    ld  hl, #_player_y
    sub a, (hl)
    jp  C, 00129$
;src/main.c:335: if (player_y+15>=collide_yFrom){
    ld  e, (hl)
    ld  d, #0x00
    ld  hl, #0x000f
    add hl,de
    ex  de,hl
    ld  hl, (_collide_yFrom)
    ld  h, #0x00
    ld  a, e
    sub a, l
    ld  a, d
    sbc a, h
    jp  PO, 00214$
    xor a, #0x80
00214$:
    jp  M, 00129$
;src/main.c:336: collide_xFrom=offsetX*8+1;
    ld  a,(#_offsetX + 0)
    rlca
    rlca
    rlca
    and a, #0xf8
    ld  hl, #_collide_xFrom
    inc a
    ld  (hl), a
;src/main.c:337: collide_xTo=collide_xFrom+7-2;
    ld  hl, #_collide_xTo
    ld  a,(#_collide_xFrom + 0)
    add a, #0x05
    ld  (hl), a
;src/main.c:339: if (player_x<=collide_xTo){
    ld  a, (#_collide_xTo)
    ld  hl, #_player_x
    sub a, (hl)
    jp  C, 00129$
;src/main.c:340: if (player_x+7>=collide_xFrom){
    ld  e, (hl)
    ld  d, #0x00
    ld  hl, #0x0007
    add hl,de
    ex  de,hl
    ld  hl, (_collide_xFrom)
    ld  h, #0x00
    ld  a, e
    sub a, l
    ld  a, d
    sbc a, h
    jp  PO, 00215$
    xor a, #0x80
00215$:
    jp  M, 00129$
;src/main.c:341: if (player_moveSpeed>0){
    xor a, a
    ld  hl, #_player_moveSpeed
    sub a, (hl)
    jp  PO, 00216$
    xor a, #0x80
00216$:
    jp  P, 00118$
;src/main.c:342: for (u8 i=0;i<5;i++){
    ld  e, #0x00
00133$:
    ld  a, e
    sub a, #0x05
    jr  NC,00129$
// ;src/main.c:343: if (tileID>=TILE_BIG_URCHIN_BEGIN && tileID<=TILE_BIG_URCHIN_END){
//     ld  hl, #_tileID
//     ld  a, (hl)
//     sub a, #0x09
//     jr  C,00109$
//     ld  a, #0x0c
//     sub a, (hl)
//     jr  C,00109$
// ;src/main.c:344: tileID=TILE_URCHIN;
//     ld  (hl), #0x08
// 00109$:
;src/main.c:347: if (tileID==TILE_URCHIN && tblCollision[i].tileID==TILE_URCHIN){
    ld  l, e
    ld  h, #0x00
    add hl, hl
    add hl, hl
    ld (_collision_tileID),hl
    // ld  a,(#_tileID + 0)
    // sub a, #0x08
    // jr  NZ,00112$
    // ld hl,(_collision_tileID)
    // add hl, bc
    // ld  a, (hl)
    // sub a, #0x08
    // jr  Z,00129$
// ;src/main.c:348: break;
// 00112$:
;src/main.c:351: if (tblCollision[i].tileID==255){
    // ld hl,(_collision_tileID)
    ld  a, #<(_tblCollision)
    add a, l
    ld  l, a
    ld  a, #>(_tblCollision)
    adc a, h
    ld  h, a
    ld (_collision_tileID),hl
    ld  d, (hl)
    inc d
    jr  NZ,00134$
;src/main.c:352: gameState=STATE_GAME_COLLISION;
    ld  hl,#_gameState + 0
    ld  (hl), #0x02
;src/main.c:353: tblCollision[i].tileID=tileID;
    ld hl,(_collision_tileID)
    ld  a,(#_tileID + 0)
    ld  (hl), a
;src/main.c:354: tblCollision[i].tileByte=tileByte;
    ld de,(_collision_tileID)
    inc de
    ld  a,(#_tileByte + 0)
    ld  (de), a
;src/main.c:355: tblCollision[i].ptrSprite=ptrSprite;
    ld de,(_collision_tileID)
    inc de
    inc de
    ld  hl, #_ptrSprite
    ld  a, (hl)
    ld  (de), a
    inc de
    inc hl
    ld  a, (hl)
    ld  (de), a
;src/main.c:356: break;
    jr  00129$
00134$:
;src/main.c:342: for (u8 i=0;i<5;i++){
    inc e
    jr  00133$
00118$:
;src/main.c:361: gameState=STATE_GAME_COLLISION;
    ld  hl,#_gameState + 0
    ld  (hl), #0x02
;src/main.c:362: return;
    ret
00129$:
;src/main.c:370: ptrSprite++;
    ld  hl, #_ptrSprite
    inc (hl)
    jr  NZ,00222$
    inc hl
    inc (hl)
00222$:
;src/main.c:316: for (offsetX=0;offsetX<=7;offsetX++){
    ld  hl, #_offsetX
    inc (hl)
    ld  a, #0x07
    sub a, (hl)
    jp  NC, 00135$
00138$:
;src/main.c:312: for (offsetY=0;offsetY<=4;offsetY++){
    ld  hl, #_offsetY
    inc (hl)
    ld  a, #0x04
    sub a, (hl)
    jp  NC, 00137$

__endasm;
}

void analyseCollisions_forward(){
    // player_zFrom=player_z;
    // player_zTo=player_z+player_moveSpeed;
    // player_z_tmp=0;

    // for (u8 z=2;z!=0;z--){
    //     ptrZone=level[zoneIndex].zone;

    //     if (ptrZone!=0){
    //         while (ptrZone->sprite!=0){
    //             if (player_zFrom<=ptrZone->z){
    //                 if (player_zTo>=ptrZone->z){
    //                     checkTileCollisions();

    //                     if (gameState==STATE_GAME_COLLISION){
    //                         if (player_z_tmp==0){
    //                             player_z_tmp=1;
    //                             player_z=ptrZone->z;
    //                         }
    //                     }
    //                 }
    //             }

    //             ptrZone++;
    //         }
    //     }

    //     zoneIndex++;
    // }

    // if (player_z_tmp!=0){
    //     player_moveSpeed++;
    //     player_moveSpeed*=-1;
    // }            

__asm

;src/main.c:377: player_zFrom=player_z;
    ld  hl, (_player_z)
    ld  (_player_zFrom), hl
;src/main.c:378: player_zTo=player_z+player_moveSpeed;
    ld a,(_player_moveSpeed)
    ld c,a
    rla
    sbc a, a
    ld  b, a
    ld  hl, #_player_zTo
    ld  de, #_player_z
    ld  a, (de)
    add a, c
    ld  (hl), a
    inc de
    ld  a, (de)
    adc a, b
    inc hl
    ld  (hl), a
;src/main.c:379: player_z_tmp=0;
    ld  hl,#_player_z_tmp + 0
    ld  (hl), #0x00
;src/main.c:381: for (u8 z=2;z!=0;z--){
    ld hl,#_z
    ld (hl),#2
00118$:
    ld a,(hl)
    or  a, a
    jr  Z, 00114$
;src/main.c:382: ptrZone=level[zoneIndex].zone;
    ld  bc, #_level+0
    ld  hl, (_zoneIndex)
    ld  h, #0x00
    add hl, hl
    add hl, bc
    ex de,hl
    ld  a, (de)
    ld  hl, #_ptrZone
    ld  (hl), a
    inc de
    ld  a, (de)
    inc hl
    ld  (hl), a
;src/main.c:384: if (ptrZone!=0){
    dec hl
    or  a,(hl)
    jr  Z,00113$
;src/main.c:385: while (ptrZone->sprite!=0){
00109$:
    ld  hl, (_ptrZone)
    ld  c, (hl)
    inc hl
    ld  b, (hl)
    dec hl
    ld  a, b
    or  a,c
    jr  Z,00113$
;src/main.c:386: if (player_zFrom<=ptrZone->z){
    inc hl
    inc hl
    ld  c, (hl)
    inc hl
    ld  b, (hl)
    ld  a, c
    ld  hl, #_player_zFrom
    sub a, (hl)
    ld  a, b
    inc hl
    sbc a, (hl)
    jr  C,00108$
;src/main.c:387: if (player_zTo>=ptrZone->z){
    ld  hl, #_player_zTo
    ld  a, (hl)
    sub a, c
    inc hl
    ld  a, (hl)
    sbc a, b
    jr  C,00108$
;src/main.c:388: checkTileCollisions();
    call    _checkTileCollisions
;src/main.c:390: if (gameState==STATE_GAME_COLLISION){
    ld  a,(#_gameState + 0)
    sub a, #0x02
    jr  NZ,00108$
;src/main.c:391: if (player_z_tmp==0){
    ld  hl, #_player_z_tmp
    ld  a, (hl)
    or  a, a
    jr  NZ,00108$
;src/main.c:392: player_z_tmp=1;
    ld  (hl), #0x01
;src/main.c:393: player_z=ptrZone->z;
    ld  hl, (_ptrZone)
    inc hl
    inc hl
    ld  a, (hl)
    ld  de, #_player_z
    ld  (de), a
    inc hl
    ld  a, (hl)
    inc de
    ld  (de), a
00108$:
;src/main.c:399: ptrZone++;
    ld  hl, #_ptrZone
    ld  a, (hl)
    add a, #0x04
    ld  (hl), a
    inc hl
    ld  a, (hl)
    adc a, #0x00
    ld  (hl), a
    jr  00109$
00113$:
;src/main.c:403: zoneIndex++;
    ld  hl, #_zoneIndex+0
    inc (hl)
;src/main.c:381: for (u8 z=2;z!=0;z--){
    ld hl,#_z
    dec (hl)
    jr  00118$
00114$:
;src/main.c:406: if (player_z_tmp!=0){
    ld  a,(#_player_z_tmp + 0)
    or  a, a
    ret z
;src/main.c:407: player_moveSpeed++;
    ld  hl, #_player_moveSpeed
    inc (hl)
;src/main.c:408: player_moveSpeed*=-1;
    xor a, a
    sub a, (hl)
    ld  (hl), a

__endasm;
}

void analyseCollisions_backward(){
    // player_zFrom=player_z+player_moveSpeed;
    // player_zTo=player_z;

    // if (zoneIndex==0)
    //     zoneReps=1;
    // else
    //     zoneReps=2;

    // for (u8 z=zoneReps;z!=0;z--){
    //     ptrZone=level[zoneIndex].zone;

    //     if (ptrZone!=0){
    //         while (ptrZone->sprite!=0){
    //             if (player_zFrom<=ptrZone->z+1){
    //                 if (player_zTo>=ptrZone->z+1){
    //                     checkTileCollisions();

    //                     if (gameState==STATE_GAME_COLLISION){
    //                         collide_behind_z=ptrZone->z;
    //                         collide_behind_zoneIndex=zoneIndex;
    //                         player_z=ptrZone->z+1;
    //                         player_moveSpeed++;
    //                         player_moveSpeed*=-1;
    //                         return;
    //                     }
    //                 }
    //             }

    //             ptrZone++;
    //         }
    //     }

    //     zoneIndex--;
    // }

__asm

;src/main.c:413: player_zFrom=player_z+player_moveSpeed;
    ld a,(_player_moveSpeed)
    ld c,a
    rla
    sbc a, a
    ld  b, a
    ld  hl, #_player_zFrom
    ld  de, #_player_z
    ld  a, (de)
    add a, c
    ld  (hl), a
    inc de
    ld  a, (de)
    adc a, b
    inc hl
    ld  (hl), a
;src/main.c:414: player_zTo=player_z;
    ld  hl, (_player_z)
    ld  (_player_zTo), hl
;src/main.c:416: if (zoneIndex==0)
    ld  a,(#_zoneIndex + 0)
    or  a, a
    jr  NZ,00102$
;src/main.c:417: zoneReps=1;
    ld  hl,#_zoneReps + 0
    ld  (hl), #0x01
    jr  00103$
00102$:
;src/main.c:419: zoneReps=2;
    ld  hl,#_zoneReps + 0
    ld  (hl), #0x02
00103$:
;src/main.c:421: for (u8 z=zoneReps;z!=0;z--){
    ld  hl,#_zoneReps + 0
    ld  c, (hl)
00117$:
    ld  a, c
    or  a, a
    ret Z
;src/main.c:422: ptrZone=level[zoneIndex].zone;
    ld  de, #_level+0
    ld  hl, (_zoneIndex)
    ld  h, #0x00
    add hl, hl
    add hl, de
    ex de,hl
    ld  a, (de)
    ld  hl, #_ptrZone
    ld  (hl), a
    inc de
    ld  a, (de)
    inc hl
    ld  (hl), a
;src/main.c:424: if (ptrZone!=0){
    dec hl
    or  a,(hl)
    jr  Z,00114$
;src/main.c:425: while (ptrZone->sprite!=0){
00110$:
    ld  de, (_ptrZone)
    ld  l, e
    ld  h, d
    ld  b, (hl)
    inc hl
    ld  a, (hl)
    or  a,b
    jr  Z,00114$
;src/main.c:426: if (player_zFrom<=ptrZone->z+1){
    ex  de,hl
    inc hl
    inc hl
    ld  e, (hl)
    inc hl
    ld  d, (hl)
    inc de
    ld  b, e
    ld  a, b
    ld  hl, #_player_zFrom
    sub a, (hl)
    ld  a, d
    inc hl
    sbc a, (hl)
    jr  C,00109$
;src/main.c:427: if (player_zTo>=ptrZone->z+1){
    ld  hl, #_player_zTo
    ld  a, (hl)
    sub a, b
    inc hl
    ld  a, (hl)
    sbc a, d
    jr  C,00109$
;src/main.c:428: checkTileCollisions();
    push    bc
    call    _checkTileCollisions
    pop bc
;src/main.c:430: if (gameState==STATE_GAME_COLLISION){
    ld  a,(#_gameState + 0)
    sub a, #0x02
    jr  NZ,00109$
;src/main.c:425: while (ptrZone->sprite!=0){
    ld  hl, (_ptrZone)
;src/main.c:426: if (player_zFrom<=ptrZone->z+1){
    inc hl
    inc hl
    ld  c, (hl)
    inc hl
    ld  b, (hl)
;src/main.c:431: collide_behind_z=ptrZone->z;
    ld  (_collide_behind_z), bc
;src/main.c:432: collide_behind_zoneIndex=zoneIndex;
    ld  a,(#_zoneIndex + 0)
    ld  hl, #_collide_behind_zoneIndex
    ld  (hl), a
;src/main.c:433: player_z=ptrZone->z+1;
    inc bc
    ld  (_player_z), bc
;src/main.c:434: player_moveSpeed++;
    ld  hl, #_player_moveSpeed
    inc (hl)
;src/main.c:435: player_moveSpeed*=-1;
    xor a, a
    sub a, (hl)
    ld  (hl), a
;src/main.c:436: return;
    ret
00109$:
;src/main.c:441: ptrZone++;
    ld  hl, #_ptrZone
    ld  a, (hl)
    add a, #0x04
    ld  (hl), a
    inc hl
    ld  a, (hl)
    adc a, #0x00
    ld  (hl), a
    jr  00110$
00114$:
;src/main.c:445: zoneIndex--;
    ld  hl, #_zoneIndex+0
    dec (hl)
;src/main.c:421: for (u8 z=zoneReps;z!=0;z--){
    dec c
    jp  00117$

__endasm;
}

void analyseCollisions(){
    clearCollisions();

    if (player_moveSpeed<player_maxSpeed){
        player_moveSpeed++;
    }
    else if (player_moveSpeed>player_maxSpeed){
        player_moveSpeed--;
    }

    if (player_moveSpeed!=0){
        zoneIndex=player_z/1024-1;

        if (player_moveSpeed>0){
            analyseCollisions_forward();
        }
        else{
            analyseCollisions_backward();
        }
    }
}

void callTileFunction()__naked{
// tblTileFunction[tileWidth-1][posX%2][tileHeight-(tileWidth-1)]();

__asm

dec a
ld (TILE_WIDTH_A+1),a
ld d,#0
ld e,a
ld h,d
ld l,e
add hl,hl
add hl,de
add hl,hl
add hl,hl

ld bc,#_tblTileFunction
add hl,bc
ld b,h
ld c,l

ld a,(_posX)
and a,#1
ld l,a
ld e,l
add hl,hl
add hl,de
add hl,hl
ld h,d
add hl,bc

ld a,(_tileHeight)
TILE_WIDTH_A:
sub a,#0xFF
add a,a
ld e,a
add hl,de
ld a,(hl)
inc hl
ld h,(hl)
ld l,a

jp (hl)

__endasm;
}

void processSprite(){
    // objectDistance=ptrZone->z-player_z;

    // if (objectDistance<1024){
    //     processSprite();
    // }

    // ptrZone++;

    // objectZoom=tblZoom[objectDistance/32][objectDistance%32];
    // posY=rectTop[objectZoom-16];
    // ptrSprite=ptrZone->sprite;
    // objectZoom--;
    // baseSize=objectZoom/8;
    // offsetRow=(objectZoom%8)*8;

    // void spriteRow(){
    // tileHeight=baseSize+offset[offsetRow+offsetY];
    // posX=63-objectZoom/2;

    // for (offsetX=0;offsetX<=7;offsetX++){
    //     tileID=*ptrSprite;
    //     ptrSprite++;

    //     if (tileID==254){
    //         break;
    //     }

    //     tileWidth=baseSize+offset[offsetRow+offsetX];
    //     drawTile();
    //     posX+=tileWidth;
    // }
    // }

    // posY+=tileHeight;

    // for (offsetY=0;offsetY<=4;offsetY++){
    //     spriteRow();
    // }

__asm

ld hl,(_ptrZone)
inc hl
inc hl
ld c,(hl)
inc hl
ld b,(hl)
ld hl,#_player_z
ld a,c
sub a,(hl)
ld e,a
ld a,b
inc hl
sbc a,(hl)
ld h,a

// sub a,#4
sub a,#2
ret nc

ld l,e
XOR A
ADD HL,HL
RLA
ADD HL,HL
RLA
ADD HL,HL
RLA
LD L,H
LD H,A

ld bc,#_tblZoom
add hl,hl
add hl,bc
ld c,(hl)
inc hl
ld b,(hl)
ld a,e
and a,#0x1f
ld l,a
ld h,#0x00
add hl,bc
ld a,(hl)
ld (_objectZoom),a

ld bc,#_rectTop
add a,#0xf0
ld l,a
ld h,#0x00
add hl,bc
ld a,(hl)
ld (_posY),a

exx
push bc

ld hl,(_ptrZone)
ld e,(hl)
inc hl
ld d,(hl)
exx

ld hl,#_objectZoom
dec (hl)

ld a,(hl)
rrca
rrca
rrca
and a,#0x1f
ld (PROCESS_SPRITE_BASE_SIZE+1),a

ld a,(hl)
and a,#7
rlca
rlca
rlca
and a,#0xf8
ld (PROCESS_SPRITE_OFFSET_ROW+1),a

.db 0xfd
ld h,#0

PROCESS_SPRITE_LOOP:

ld bc,#_offset
PROCESS_SPRITE_OFFSET_ROW:
ld a,#0xff
ld (SPRITE_ROW_OFFSET_ROW+1),a
.db 0xfd
add a,h
ld l,a
ld h,#0
add hl,bc
PROCESS_SPRITE_BASE_SIZE:
ld a,#0xff
ld (SPRITE_ROW_BASE_SIZE+1),a
add a,(hl)
ld (_tileHeight),a

ld hl,#_objectZoom
ld e,(hl)
srl e
ld a,#0x3f
sub a,e
ld (_posX),a

.db 0xfd
ld l,#0

SPRITE_ROW_LOOP:

exx
ld a,(de)
inc de
exx

bit 7,a
jr nz,SPRITE_ROW_END
ld e,a

ld bc,#_offset
SPRITE_ROW_OFFSET_ROW:
ld a,#0xff
.db 0xfd
add a,l
ld l,a
ld h,#0
add hl,bc
SPRITE_ROW_BASE_SIZE:
ld a,#0xff
add a,(hl)
ld (SKIP_TILE+1),a
ld d,a

ld a,e
bit 4,a
jr nz,FOOD_ACTIVE

bit 6,a
jr z,SKIP_TILE

and a,#0b00001111 // isolate right nibble to get tileID
jr PRINT_TILE

FOOD_ACTIVE:
and a,#0b00001111 // isolate right nibble to get tileID
add a,#16 // food tiles are stored 16 bytes after block tiles

PRINT_TILE:
ld (_tileID),a
ld a,d
call _callTileFunction

SKIP_TILE:
ld a,#0xFF
ld hl,#_posX
add a,(hl)
ld (hl),a

.db 0xfd
inc l
ld a,#7
.db 0xfd
sub a,l
jr nc,SPRITE_ROW_LOOP

SPRITE_ROW_END:
ld a,(_tileHeight)
ld hl,#_posY
add a,(hl)
ld (hl),a

.db 0xfd
inc h
ld a,#4
.db 0xfd
sub a,h
jr nc,PROCESS_SPRITE_LOOP

exx
pop bc
exx

__endasm;
}

void populateVmemPlayer(){
__asm

ld c,a

ld a,(_page)
sub a,#0xc0
jr nz,VMEM_PLAYER_A

ld de,#_yAddr80
jr VMEM_PLAYER_B

VMEM_PLAYER_A:

ld de,#_yAddrC0

VMEM_PLAYER_B:

ld hl,(_player_y)
ld h,#0x00
add hl,hl
add hl,de
ex de,hl

ld a,(de)
inc e
add a,c
ld l,a
ld a,(de)
inc e
ld h,a
ld (_vmemPlayer+0),hl
inc e
inc e

ld a,(de)
inc e
add a,c
ld l,a
ld a,(de)
inc e
ld h,a
ld (_vmemPlayer+2),hl
inc e
inc e

ld a,(de)
inc e
add a,c
ld l,a
ld a,(de)
inc e
ld h,a
ld (_vmemPlayer+4),hl
inc e
inc e

ld a,(de)
inc e
add a,c
ld l,a
ld a,(de)
inc e
ld h,a
ld (_vmemPlayer+6),hl
inc e
inc e

ld a,(de)
inc e
add a,c
ld l,a
ld a,(de)
inc e
ld h,a
ld (_vmemPlayer+8),hl
inc e
inc e

ld a,(de)
inc e
add a,c
ld l,a
ld a,(de)
inc e
ld h,a
ld (_vmemPlayer+10),hl
inc e
inc e

ld a,(de)
inc e
add a,c
ld l,a
ld a,(de)
inc e
ld h,a
ld (_vmemPlayer+12),hl
inc e
inc e

ld a,(de)
inc e
add a,c
ld l,a
ld a,(de)
ld h,a
ld (_vmemPlayer+14),hl

__endasm;
}

void drawPlayerRow_left(){
__asm

ld a,(hl)
rrca
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
rlca
ld c,a
ld a,(hl)
and a,#ISOLATE_LEFT
rrca
or a,c
ld c,a
inc l
ld a,(de)
and a,b
or a,c
ld (de),a
dec e

ld a,(hl)
rrca
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
rlca
ld c,a
ld a,(hl)
and a,#ISOLATE_LEFT
rrca
or a,c
ld c,a
inc l
ld a,(de)
and a,b
or a,c
ld (de),a
dec e

ld a,(hl)
rrca
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
rlca
ld c,a
ld a,(hl)
and a,#ISOLATE_LEFT
rrca
or a,c
ld c,a
inc l
ld a,(de)
and a,b
or a,c
ld (de),a
dec e

ld a,(hl)
rrca
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
rlca
ld c,a
ld a,(hl)
and a,#ISOLATE_LEFT
rrca
or a,c
ld c,a
inc l
ld a,(de)
and a,b
or a,c
ld (de),a
dec e

ld a,(hl)
rrca
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
rlca
ld c,a
ld a,(hl)
and a,#ISOLATE_LEFT
rrca
or a,c
ld c,a
inc l
ld a,(de)
and a,b
or a,c
ld (de),a
dec e

ld a,(hl)
rrca
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
rlca
ld c,a
ld a,(hl)
and a,#ISOLATE_LEFT
rrca
or a,c
ld c,a
inc l
ld a,(de)
and a,b
or a,c
ld (de),a
dec e

ld a,(hl)
rrca
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
rlca
ld c,a
ld a,(hl)
and a,#ISOLATE_LEFT
rrca
or a,c
ld c,a
inc l
ld a,(de)
and a,b
or a,c
ld (de),a
dec e

ld a,(hl)
rrca
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
rlca
ld c,a
ld a,(hl)
and a,#ISOLATE_LEFT
rrca
or a,c
ld c,a
inc l
ld a,(de)
and a,b
or a,c
ld (de),a

ex de,hl
ld bc,#0x1007
add hl,bc
ex de,hl

ld a,(hl)
rrca
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
rlca
ld c,a
ld a,(hl)
and a,#ISOLATE_LEFT
rrca
or a,c
ld c,a
inc l
ld a,(de)
and a,b
or a,c
ld (de),a
dec e

ld a,(hl)
rrca
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
rlca
ld c,a
ld a,(hl)
and a,#ISOLATE_LEFT
rrca
or a,c
ld c,a
inc l
ld a,(de)
and a,b
or a,c
ld (de),a
dec e

ld a,(hl)
rrca
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
rlca
ld c,a
ld a,(hl)
and a,#ISOLATE_LEFT
rrca
or a,c
ld c,a
inc l
ld a,(de)
and a,b
or a,c
ld (de),a
dec e

ld a,(hl)
rrca
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
rlca
ld c,a
ld a,(hl)
and a,#ISOLATE_LEFT
rrca
or a,c
ld c,a
inc l
ld a,(de)
and a,b
or a,c
ld (de),a
dec e

ld a,(hl)
rrca
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
rlca
ld c,a
ld a,(hl)
and a,#ISOLATE_LEFT
rrca
or a,c
ld c,a
inc l
ld a,(de)
and a,b
or a,c
ld (de),a
dec e

ld a,(hl)
rrca
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
rlca
ld c,a
ld a,(hl)
and a,#ISOLATE_LEFT
rrca
or a,c
ld c,a
inc l
ld a,(de)
and a,b
or a,c
ld (de),a
dec e

ld a,(hl)
rrca
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
rlca
ld c,a
ld a,(hl)
and a,#ISOLATE_LEFT
rrca
or a,c
ld c,a
inc l
ld a,(de)
and a,b
or a,c
ld (de),a
dec e

ld a,(hl)
rrca
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
rlca
ld c,a
ld a,(hl)
and a,#ISOLATE_LEFT
rrca
or a,c
ld c,a
inc l
ld a,(de)
and a,b
or a,c
ld (de),a

__endasm;
}

void drawPlayerRow_right(){
__asm

ld a,(de)
and a,(hl)
inc l
or a,(hl)
inc l
ld (de),a
inc e

ld a,(de)
and a,(hl)
inc l
or a,(hl)
inc l
ld (de),a
inc e

ld a,(de)
and a,(hl)
inc l
or a,(hl)
inc l
ld (de),a
inc e

ld a,(de)
and a,(hl)
inc l
or a,(hl)
inc l
ld (de),a
inc e

ld a,(de)
and a,(hl)
inc l
or a,(hl)
inc l
ld (de),a
inc e

ld a,(de)
and a,(hl)
inc l
or a,(hl)
inc l
ld (de),a
inc e

ld a,(de)
and a,(hl)
inc l
or a,(hl)
inc l
ld (de),a
inc e

ld a,(de)
and a,(hl)
inc l
or a,(hl)
inc l
ld (de),a

ex de,hl
ld bc,#0x1000-7
add hl,bc
ex de,hl

ld a,(de)
and a,(hl)
inc l
or a,(hl)
inc l
ld (de),a
inc e

ld a,(de)
and a,(hl)
inc l
or a,(hl)
inc l
ld (de),a
inc e

ld a,(de)
and a,(hl)
inc l
or a,(hl)
inc l
ld (de),a
inc e

ld a,(de)
and a,(hl)
inc l
or a,(hl)
inc l
ld (de),a
inc e

ld a,(de)
and a,(hl)
inc l
or a,(hl)
inc l
ld (de),a
inc e

ld a,(de)
and a,(hl)
inc l
or a,(hl)
inc l
ld (de),a
inc e

ld a,(de)
and a,(hl)
inc l
or a,(hl)
inc l
ld (de),a
inc e

ld a,(de)
and a,(hl)
inc l
or a,(hl)
inc l
ld (de),a

__endasm;
}

void drawPlayerAllRows_left(){
__asm

ld de,(_vmemPlayer+0)
call _drawPlayerRow_left
ld de,(_vmemPlayer+2)
call _drawPlayerRow_left
ld de,(_vmemPlayer+4)
call _drawPlayerRow_left
ld de,(_vmemPlayer+6)
call _drawPlayerRow_left
ld de,(_vmemPlayer+8)
call _drawPlayerRow_left
ld de,(_vmemPlayer+10)
call _drawPlayerRow_left
ld de,(_vmemPlayer+12)
call _drawPlayerRow_left
ld de,(_vmemPlayer+14)
jp _drawPlayerRow_left

__endasm;
}

void drawPlayerAllRows_right(){
__asm

ld de,(_vmemPlayer+0)
call _drawPlayerRow_right
ld de,(_vmemPlayer+2)
call _drawPlayerRow_right
ld de,(_vmemPlayer+4)
call _drawPlayerRow_right
ld de,(_vmemPlayer+6)
call _drawPlayerRow_right
ld de,(_vmemPlayer+8)
call _drawPlayerRow_right
ld de,(_vmemPlayer+10)
call _drawPlayerRow_right
ld de,(_vmemPlayer+12)
call _drawPlayerRow_right
ld de,(_vmemPlayer+14)
jp _drawPlayerRow_right

__endasm;
}

void drawPlayerSprite_left(){
__asm

ld a,(_player_x)
add a,#7
call _populateVmemPlayer

ld hl,(_player_frame)
ld h,#0x00
add hl,hl
ld bc,#_playerFrames
add hl,bc
ld a,(hl)
inc l
ld h,(hl)
ld l,a

jp _drawPlayerAllRows_left

__endasm;
}

void drawPlayerSprite_right(){
__asm

ld a,(_player_x)
call _populateVmemPlayer

ld hl,(_player_frame)
ld h,#0x00
add hl,hl
ld bc,#_playerFrames
add hl,bc
ld a,(hl)
inc l
ld h,(hl)
ld l,a

jp _drawPlayerAllRows_right

__endasm;
}

void drawPlayerRed_left(){
__asm

ld a,(_player_x)
add a,#7
call _populateVmemPlayer

ld hl,#_playerRed

jp _drawPlayerAllRows_left

__endasm;
}

void drawPlayerRed_right(){
__asm

ld a,(_player_x)
call _populateVmemPlayer

ld hl,#_playerRed

jp _drawPlayerAllRows_right

__endasm;
}

void drawPlayer(){
    if (player_dead==0){
        if (player_direction==PLAYER_DIRECTION_LEFT){
            drawPlayerSprite_left();
        }    
        else{
            drawPlayerSprite_right();
        }

        if (player_frame==PLAYER_FRAME_LIMIT){
            player_frame=0;
        }
        else{
            player_frame++;
        }
    }
    else if (player_dead==1){
        if (player_direction==PLAYER_DIRECTION_LEFT){
            drawPlayerRed_left();
        }    
        else{
            drawPlayerRed_right();
        }
    }
    else if (player_dead==2){
        player_frame=0;

        if (player_direction==PLAYER_DIRECTION_LEFT){
            drawPlayerSprite_left();
        }    
        else{
            drawPlayerSprite_right();
        }
    }
}

void drawSprites(){
    // ptrLevel=tblLevel[currentLevel];
    // zoneIndex=player_z/1024;

    // for (u8 i=0;i<2;i++){
    //     if (zoneIndex<=63){
    //         ptrZone=ptrLevel[zoneIndex].zone;

            // if (ptrZone!=NULL){
            //     while (ptrZone->sprite!=NULL){
            //         processSprite();
            //     }
            // }
    //     }

    //     zoneIndex--;
    // }

__asm

di

// ld bc,#_tblLevel
// ld hl,(_currentLevel)
// ld h,#0
// add hl,hl
// add hl,bc
// ld a,(hl)
// inc hl
// ld h,(hl)
// ld l,a
// // ld (_ptrLevel),hl
// // ld (DRAW_SPRITES_PTR_LEVEL+1),hl

// ld hl,#_level
// ld (DRAW_SPRITES_PTR_LEVEL+1),hl

ld a,(_player_z+1)
rrca
rrca
and a,#0x3f
ld (_zoneIndex),a

// ld hl,(_ptrLevel)
// ld (DRAW_SPRITES_PTR_LEVEL+1),hl

ld c,#2

DRAW_SPRITES_LOOP:
ld a,#0x3f
ld hl,#_zoneIndex
sub a,(hl)
jr c,DRAW_SPRITES_A

push bc

ld l,(hl)
ld h,#0
add hl,hl
// DRAW_SPRITES_PTR_LEVEL:
// ld de,#0xffff
ld de,#_level
add hl,de
ld a,(hl)
inc hl
ld h,(hl)
ld l,a
ld (_ptrZone),hl

ld a,h
or a,l
jr z,ZONE_LOOP_END

ZONE_LOOP:
ld a,(hl)
inc hl
or a,(hl)
jr z,ZONE_LOOP_END

call _processSprite

ld hl,(_ptrZone)
ld bc,#4
add hl,bc
ld (_ptrZone),hl
jr ZONE_LOOP

ZONE_LOOP_END:

pop bc

DRAW_SPRITES_A:
ld hl,#_zoneIndex
dec (hl)

dec c
jr nz,DRAW_SPRITES_LOOP

ei

__endasm;
}

void rectA(){
    // p=pLineTop;

    // for (u8 b=lineWidth/2;b!=0;b--){
    //     *p=RECT_BYTE;
    //     p++;
    // }

    __asm

    ld hl,(_pLineTop)
    ld d,h
    ld e,l
    inc e

    ld bc,(_lineWidth)
    ld b,#0
    srl c
    dec c

    ld (hl),#RECT_BYTE
    ldir

    // p=pLineBottom;

    // for (u8 b=lineWidth/2;b!=0;b--){
    //     *p=RECT_BYTE;
    //     p++;
    // }

    ld hl,(_pLineBottom)
    ld d,h
    ld e,l
    inc e

    ld bc,(_lineWidth)
    ld b,#0
    srl c
    dec c

    ld (hl),#RECT_BYTE
    ldir

    // for (u8 b=lineBottom-lineTop-2;b!=0;b--){
    //     p=*pLineY+lineLeft/2;
    //     *p=RECT_LEFT;
    //     p+=lineWidth/2-1;
    //     *p=RECT_RIGHT;
    //     pLineY++;
    // }

    ld a,(_lineLeft)
    ld b,a
    srl b

    ld a,(_lineWidth)
    ld c,a
    srl c
    dec c

    ld d,#RECT_LEFT
    ld e,#RECT_RIGHT

    exx
    push bc

    ld hl,#_lineTop
    ld a,(_lineBottom)
    sub a,(hl)
    ld b,a
    dec b
    dec b

    ld hl,(_pLineY)

    RECT_VERTA_LOOP:

    ld a,(hl)
    inc l
    ex af,af
    ld a,(hl)
    inc l
    exx

    ld h,a
    ex af,af

    add a,b
    ld l,a
    ld (hl),d

    add a,c
    ld l,a
    ld (hl),e

    exx
    djnz RECT_VERTA_LOOP
    pop bc
    exx

    __endasm;
}

void rectB(){
    // p=pLineTop;

    // for (u8 b=lineWidth/2;b!=0;b--){
    //     *p=RECT_BYTE;
    //     p++;
    // }

    // *p=RECT_LEFT;

    __asm

    ld hl,(_pLineTop)
    ld d,h
    ld e,l
    inc e

    ld bc,(_lineWidth)
    ld b,#0
    srl c
    dec c

    ld (hl),#RECT_BYTE
    ldir

    ex de,hl
    ld (hl),#RECT_LEFT

    // p=pLineBottom;

    // for (u8 b=lineWidth/2;b!=0;b--){
    //     *p=RECT_BYTE;
    //     p++;
    // }

    // *p=RECT_LEFT;

    ld hl,(_pLineBottom)
    ld d,h
    ld e,l
    inc e

    ld bc,(_lineWidth)
    ld b,#0
    srl c
    dec c

    ld (hl),#RECT_BYTE
    ldir

    ex de,hl
    ld (hl),#RECT_LEFT

    // for (u8 b=lineBottom-lineTop-2;b!=0;b--){
    //     p=*pLineY+lineLeft/2;
    //     *p=RECT_LEFT;
    //     p+=lineWidth/2;
    //     *p=RECT_LEFT;
    //     pLineY++;
    // }

    ld a,(_lineLeft)
    ld b,a
    srl b

    ld a,(_lineWidth)
    ld c,a
    srl c

    ld d,#RECT_LEFT

    exx
    push bc

    ld hl,#_lineTop
    ld a,(_lineBottom)
    sub a,(hl)
    ld b,a
    dec b
    dec b

    ld hl,(_pLineY)

    RECT_VERTB_LOOP:

    ld a,(hl)
    inc l
    ex af,af
    ld a,(hl)
    inc l
    exx

    ld h,a
    ex af,af

    add a,b
    ld l,a
    ld (hl),d

    add a,c
    ld l,a
    ld (hl),d

    exx
    djnz RECT_VERTB_LOOP
    pop bc
    exx

    __endasm;
}

void rectC(){
    // p=pLineTop;
    // *p=RECT_RIGHT;
    // p++;

    // for (u8 b=lineWidth/2-1;b!=0;b--){
    //     *p=RECT_BYTE;
    //     p++;
    // }

    // *p=RECT_LEFT;

    __asm

    ld hl,(_pLineTop)
    ld (hl),#RECT_RIGHT
    inc l

    ld d,h
    ld e,l
    inc e

    ld bc,(_lineWidth)
    ld b,#0
    srl c
    dec c
    dec c

    ld (hl),#RECT_BYTE
    ldir

    ex de,hl
    ld (hl),#RECT_LEFT

    // p=pLineBottom;
    // *p=RECT_RIGHT;
    // p++;

    // for (u8 b=lineWidth/2-1;b!=0;b--){
    //     *p=RECT_BYTE;
    //     p++;
    // }

    // *p=RECT_LEFT;

    ld hl,(_pLineBottom)
    ld (hl),#RECT_RIGHT
    inc l

    ld d,h
    ld e,l
    inc e

    ld bc,(_lineWidth)
    ld b,#0
    srl c
    dec c
    dec c

    ld (hl),#RECT_BYTE
    ldir

    ex de,hl
    ld (hl),#RECT_LEFT

    // for (u8 b=lineBottom-lineTop-2;b!=0;b--){
    //     p=*pLineY+lineLeft/2;
    //     *p=RECT_RIGHT;
    //     p+=lineWidth/2;
    //     *p=RECT_LEFT;
    //     pLineY++;
    // }

    ld a,(_lineLeft)
    ld b,a
    srl b

    ld a,(_lineWidth)
    ld c,a
    srl c

    ld d,#RECT_RIGHT
    ld e,#RECT_LEFT

    exx
    push bc

    ld hl,#_lineTop
    ld a,(_lineBottom)
    sub a,(hl)
    ld b,a
    dec b
    dec b

    ld hl,(_pLineY)

    RECT_VERTC_LOOP:

    ld a,(hl)
    inc l
    ex af,af
    ld a,(hl)
    inc l
    exx

    ld h,a
    ex af,af

    add a,b
    ld l,a
    ld (hl),d

    add a,c
    ld l,a
    ld (hl),e

    exx
    djnz RECT_VERTC_LOOP
    pop bc
    exx

    __endasm;
}

void rectD(){
    // p=pLineTop;
    // *p=RECT_RIGHT;
    // p++;

    // for (u8 b=lineWidth/2;b!=0;b--){
    //     *p=RECT_BYTE;
    //     p++;
    // }

    __asm

    ld hl,(_pLineTop)
    ld (hl),#RECT_RIGHT
    inc l

    ld d,h
    ld e,l
    inc e

    ld bc,(_lineWidth)
    ld b,#0
    srl c
    dec c

    ld (hl),#RECT_BYTE
    ldir

    // p=pLineBottom;
    // *p=RECT_RIGHT;
    // p++;

    // for (u8 b=lineWidth/2;b!=0;b--){
    //     *p=RECT_BYTE;
    //     p++;
    // }

    ld hl,(_pLineBottom)
    ld (hl),#RECT_RIGHT
    inc l

    ld d,h
    ld e,l
    inc e

    ld bc,(_lineWidth)
    ld b,#0
    srl c
    dec c

    ld (hl),#RECT_BYTE
    ldir

    // for (u8 b=lineBottom-lineTop-2;b!=0;b--){
    //     p=*pLineY+lineLeft/2;
    //     *p=RECT_RIGHT;
    //     p+=lineWidth/2;
    //     *p=RECT_RIGHT;
    //     pLineY++;
    // }

    ld a,(_lineLeft)
    ld b,a
    srl b

    ld a,(_lineWidth)
    ld c,a
    srl c

    ld d,#RECT_RIGHT

    exx
    push bc

    ld hl,#_lineTop
    ld a,(_lineBottom)
    sub a,(hl)
    ld b,a
    dec b
    dec b

    ld hl,(_pLineY)

    RECT_VERTD_LOOP:

    ld a,(hl)
    inc l
    ex af,af
    ld a,(hl)
    inc l
    exx

    ld h,a
    ex af,af

    add a,b
    ld l,a
    ld (hl),d

    add a,c
    ld l,a
    ld (hl),d

    exx
    djnz RECT_VERTD_LOOP
    pop bc
    exx

    __endasm;
}

void drawRects(){
    // for (rectCount=1;rectCount<=2;rectCount++){

    // rectZ=(((player_z/512)+rectCount)*512)-1;
    // objectDistance=rectZ-player_z;
    // objectZoom=tblZoom[objectDistance/32][objectDistance%32];
    // lineWidth=objectZoom;
    // lineTop=rectTop[objectZoom-16];
    // lineBottom=rectBottom[objectZoom-16];
    // objectZoom--;
    // lineLeft=63-objectZoom/2;
    // lineRight=lineLeft+objectZoom;
    // pLineTop=*(yAddr+lineTop)+lineLeft/2;
    // pLineBottom=*(yAddr+lineBottom-1)+lineLeft/2;
    // pLineY=yAddr+lineTop+1;

    // if (lineLeft%2==0){
    //     if (lineWidth%2==0){
    //         rectA();
    //     }
    //     else{
    //         rectB();
    //     }
    // }
    // else{
    //     if (lineWidth%2==0){
    //         rectC();
    //     }
    //     else{
    //         rectD();
    //     }
    // }

    // }

__asm

di

ld hl,#_rectCount
ld (hl),#1

DRAW_RECTS_LOOP:

ld de,(_player_z)
ld a,d
srl a
ld hl,#_rectCount
add a,(hl)
ld h,#0
ld l,a
add a,a
ld b,a
ld a,h
add a,#0xff
ld l,a
ld a,b
adc a,#0xff
ld h,a
or a
sbc hl,de
ld e,l

XOR A
ADD HL,HL
RLA
ADD HL,HL
RLA
ADD HL,HL
RLA
LD L,H
LD H,A

ld bc,#_tblZoom
add hl,hl
add hl,bc
ld c,(hl)
inc hl
ld b,(hl)
ld a,e
and a,#0x1f
ld l,a
ld h,#0x00
add hl,bc
ld a,(hl)
ld (_objectZoom),a
ld e,a

ld (_lineWidth),a

ld bc,#_rectTop
add a,#0xf0
ld l,a
ld h,#0
add hl,bc
ld a,(hl)
ld (_lineTop),a

ld bc,#_rectBottom
ld a,e
add a,#0xf0
ld l,a
ld h,#0
add hl,bc
ld a,(hl)
ld (_lineBottom),a

ld hl,#_objectZoom
dec (hl)

ld c,(hl)
srl c
ld a,#0x3f
sub a,c
ld (_lineLeft),a

add a,(hl)
ld (_lineRight),a

ld hl,(_lineTop)
ld h,#0
add hl,hl
ld de,(_yAddr)
add hl,de
ld c,(hl)
inc hl
ld b,(hl)
ld hl,#_lineLeft
ld e,(hl)
srl e
ld a,c
ld hl,#_pLineTop
add a,e
ld (hl),a
ld a,b
adc a,#0x00
inc hl
ld (hl),a

ld hl,(_lineBottom)
ld h,#0
add hl,hl
ld de,(_yAddr)
add hl,de
dec hl
dec hl
ld c,(hl)
inc hl
ld b,(hl)
ld hl,#_lineLeft
ld e,(hl)
srl e
ld a,c
ld hl,#_pLineBottom
add a,e
ld (hl),a
ld a,b
adc a,#0x00
inc hl
ld (hl), a

ld hl,(_lineTop)
ld h,#0
add hl,hl
ld de,(_yAddr)
ld a,e
add a,l
ld l,a
ld a,d
adc a,h
ld h,a
inc hl
inc hl
ld (_pLineY),hl

ld a,(_lineLeft)
and a,#0x01
ld c,a
ld a,(_lineWidth)
and a,#0x01
ld b,a
ld a,c
or a,a
jr nz,DRAW_RECTS_A

or a,b
jr nz,DRAW_RECTS_B
call _rectA
jr DRAW_RECTS_D

DRAW_RECTS_B:
call _rectB
jr DRAW_RECTS_D

DRAW_RECTS_A:
ld a,b
or a,a
jr nz,DRAW_RECTS_C
call _rectC
jr DRAW_RECTS_D

DRAW_RECTS_C:
call _rectD

DRAW_RECTS_D:
ld hl,#_rectCount
inc (hl)
ld a,#0x02
sub a,(hl)
jp nc,DRAW_RECTS_LOOP

ei

__endasm;
}

void populateYaddress(){
    u8* vmem80;
    u8* vmemC0;
    u8 i;

    for (u8 y=0;y<21;y++){
        vmem80=SCREEN_BUFFER+y*64;
        vmemC0=CPCT_VMEM_START+y*64;
        i=y*4;

        for (u8 j=0;j<4;j++){
            yAddr80[i+j]=vmem80;
            yAddrC0[i+j]=vmemC0;

            vmem80+=0x1000;
            vmemC0+=0x1000;
        }
    }
}

void clsStack(){
__asm

ld (SP_CLS_STACK+1),sp
ld sp,(_p)

ld hl,(_clsColour)

ld a,(_clsReps)
ld b,a

LOOP_A_4:

push hl
push hl
push hl
push hl
push hl
push hl
push hl
push hl

push hl
push hl
push hl
push hl
push hl
push hl
push hl
push hl

push hl
push hl
push hl
push hl
push hl
push hl
push hl
push hl

push hl
push hl
push hl
push hl
push hl
push hl
push hl
push hl

djnz LOOP_A_4

SP_CLS_STACK:
ld sp,#0xFFFF

__endasm;
}

void cls24(u16 useColour)__z88dk_fastcall{
    // p=vmem+24*64;
    // clsColour=#0x0000;
    // clsReps=24;

    // for (u8 b=8;b!=0;b--){
    //     clsStack();
    //     p+=0x0800;
    // }

    clsColour=useColour;

__asm

di

ld hl,(_vmem)
ld de,#24*64
add hl,de
ld (_p),hl

ld hl,#_clsReps
ld (hl),#24

ld c,#8

CLS24_LOOP:

push bc
call _clsStack
pop bc

ld hl,(_p)
ld de,#0x0800
add hl,de
ld (_p),hl

dec c
jr nz,CLS24_LOOP

ei

__endasm;
}

void cls20(){
    // p=vmem+20*64;
    // clsColour=#0x0000;
    // clsReps=20;

    // for (u8 b=4;b!=0;b--){
    //     clsStack();
    //     p+=0x1000;
    // }

__asm

di

ld hl,#CLS_BLACK
ld (_clsColour),hl

ld hl,(_vmem)
ld de,#20*64
add hl,de
ld (_p),hl

ld hl,#_clsReps
ld (hl),#20

ld c,#4

CLS20_LOOP:

push bc
call _clsStack
pop bc

ld hl,(_p)
ld de,#0x1000
add hl,de
ld (_p),hl

dec c
jr nz,CLS20_LOOP

ei

__endasm;
}

// void drawScreen(){
// __asm

// di
// ld bc,#0x0000
// call _drawScreenRow
// ld bc,#0x1000
// call _drawScreenRow
// ld bc,#0x2000
// call _drawScreenRow
// ld bc,#0x3000
// call _drawScreenRow
// ei

// __endasm;
// }

// void drawScreenRow(){
// __asm

// ld hl,(_vmem)
// add hl,bc

// ld (SAVE_SP_DRAW_SCREEN+1),sp
// ld sp,hl

// ld hl,#_i
// ld (hl),#20

// DRAW_SCREEN_BLOCK1:

// pop hl
// pop de
// pop bc
// pop ix
// pop iy
// pop af
// exx
// pop de
// pop bc

// ex af,af
// ld hl,#0x0800
// add hl,sp
// ld sp,hl
// ex af,af

// push bc
// push de
// exx
// push af
// push iy
// push ix
// push bc
// push de
// push hl

// ld hl,#-0x0800+16
// add hl,sp
// ld sp,hl

// DRAW_SCREEN_BLOCK2:

// pop hl
// pop de
// pop bc
// pop ix
// pop iy
// pop af
// exx
// pop de
// pop bc

// ex af,af
// ld hl,#0x0800
// add hl,sp
// ld sp,hl
// ex af,af

// push bc
// push de
// exx
// push af
// push iy
// push ix
// push bc
// push de
// push hl

// ld hl,#-0x0800+16
// add hl,sp
// ld sp,hl

// DRAW_SCREEN_BLOCK3:

// pop hl
// pop de
// pop bc
// pop ix
// pop iy
// pop af
// exx
// pop de
// pop bc

// ex af,af
// ld hl,#0x0800
// add hl,sp
// ld sp,hl
// ex af,af

// push bc
// push de
// exx
// push af
// push iy
// push ix
// push bc
// push de
// push hl

// ld hl,#-0x0800+16
// add hl,sp
// ld sp,hl

// DRAW_SCREEN_BLOCK4:

// pop hl
// pop de
// pop bc
// pop ix
// pop iy
// pop af
// exx
// pop de
// pop bc

// ex af,af
// ld hl,#0x0800
// add hl,sp
// ld sp,hl
// ex af,af

// push bc
// push de
// exx
// push af
// push iy
// push ix
// push bc
// push de
// push hl

// ld hl,#-0x0800+16
// add hl,sp
// ld sp,hl

// ld hl,#_i
// dec (hl)
// jp nz,DRAW_SCREEN_BLOCK1

// SAVE_SP_DRAW_SCREEN:
// ld sp,#0xffff

// __endasm;
// }

void drawScreen(){
__asm

di

ld hl,(_vmem)
ld (_p),hl

.db 0xfd
ld l,#4

LOOP_A_3:

ld a,#8
add a,h
ld d,a
ld e,l

ld a,#255
ld b,#32

LOOP_A_2:

ld c,a

ldi
ldi
ldi
ldi
ldi
ldi
ldi
ldi

ldi
ldi
ldi
ldi
ldi
ldi
ldi
ldi

ldi
ldi
ldi
ldi
ldi
ldi
ldi
ldi

ldi
ldi
ldi
ldi
ldi
ldi
ldi
ldi

ldi
ldi
ldi
ldi
ldi
ldi
ldi
ldi

djnz LOOP_A_2

ld hl,(_p)
ld a,#16
add a,h
ld h,a
ld (_p),hl

.db 0xfd
dec l
jr nz,LOOP_A_3

ei

__endasm;
}

void prepareTile_setPage(){
__asm
ld bc,(_yAddr)
ld a,b
ld (PREPARE_TILE_SET_PAGE+1),a
__endasm;
}

void changeVideoMemoryPage(){
    cpct_waitVSYNC();

    if (page==0xC0){
        cpct_setVideoMemoryPage(cpct_page80);
        page=0x80;
        yAddr=yAddrC0;
        vmem=CPCT_VMEM_START;
    }
    else{
        cpct_setVideoMemoryPage(cpct_pageC0);
        page=0xC0;
        yAddr=yAddr80;
        vmem=SCREEN_BUFFER;
    }

    prepareTile_setPage();
}

void textPage(){
    silence();
    textDone=0;

    cpct_waitVSYNC();
    cpct_setPalette(hw_palette_black,16);
    cpct_setBorder(HW_BLACK);

    cpct_setVideoMemoryPage(cpct_pageC0);
    vmem=CPCT_VMEM_START;
    cls24(CLS_BLACK);

    cpct_waitVSYNC();
    cpct_setPalette(hw_palette_colour,16);
}

void decrunchLevel(){
__asm

ld a,(_currentLevel)
cp #2
jr z,DECRUNCH_LEVEL_3
cp #1
jr z,DECRUNCH_LEVEL_2

ld hl,#_level1_lz_bin
jr DECRUNCH_LEVEL

DECRUNCH_LEVEL_2:
ld hl,#_level2_lz_bin
jr DECRUNCH_LEVEL

DECRUNCH_LEVEL_3:
ld hl,#_level3_lz_bin

DECRUNCH_LEVEL:
ld de,#0x1000
push ix
push iy
call _deexo
pop iy
pop ix

__endasm;
}

void relocateData(){
    __asm

    ld hl,#_hw_palette_black_tmp
    ld de,#_hw_palette_black
    ld bc,#16
    ldir

    ld hl,#_tblTileLocation_0_tmp
    ld de,#_tblTileLocation_0
    ld bc,#48
    ldir

    ld hl,#_asciiChar_tmp
    ld de,#_asciiChar
    ld bc,#20
    ldir

    ld hl,#_hw_palette_colour_tmp
    ld de,#_hw_palette_colour
    ld bc,#16
    ldir

    ld hl,#_tblTileFunction_tmp
    ld de,#_tblTileFunction
    ld bc,#192
    ldir

    ld hl,#_tblZoom_tmp
    ld de,#_tblZoom
    ld bc,#64
    ldir

    __endasm;
}

void decrunchData(){
__asm

ld hl,#_offset_bin
ld de,#_offset
call _LZX0_decrunch

ld hl,#_player0_bin
ld de,#_player0
call _LZX0_decrunch

ld hl,#_player1_bin
ld de,#_player1
call _LZX0_decrunch

ld hl,#_player2_bin
ld de,#_player2
call _LZX0_decrunch

ld hl,#_player3_bin
ld de,#_player3
call _LZX0_decrunch

ld hl,#_player4_bin
ld de,#_player4
call _LZX0_decrunch

ld hl,#_playerRed_bin
ld de,#_playerRed
call _LZX0_decrunch

ld hl,#_rectTop_bin
ld de,#_rectTop
call _LZX0_decrunch

ld hl,#_sprite0_sprite1_sprite2_0_bin
ld de,#0x8600
call _LZX0_decrunch

ld hl,#_sprite0_sprite1_sprite2_1_bin
ld de,#0xc600
call _LZX0_decrunch

ld hl,#_sprite3_sprite4_sprite5_0_bin
ld de,#0x8e00
call _LZX0_decrunch

ld hl,#_sprite3_sprite4_sprite5_1_bin
ld de,#0xce00
call _LZX0_decrunch

ld hl,#_sprite6_sprite7_sprite8_0_bin
ld de,#0x9600
call _LZX0_decrunch

ld hl,#_sprite6_sprite7_sprite8_1_bin
ld de,#0xd600
call _LZX0_decrunch

ld hl,#_sprite9_sprite10_sprite11_0_bin
ld de,#0x9e00
call _LZX0_decrunch

ld hl,#_sprite9_sprite10_sprite11_1_bin
ld de,#0xde00
call _LZX0_decrunch

ld hl,#_sprite12_sprite13_sprite14_0_bin
ld de,#0xa600
call _LZX0_decrunch

ld hl,#_sprite12_sprite13_sprite14_1_bin
ld de,#0xe600
call _LZX0_decrunch

ld hl,#_sprite15_food0_food1_0_bin
ld de,#0xae00
call _LZX0_decrunch

ld hl,#_sprite15_food0_food1_1_bin
ld de,#0xee00
call _LZX0_decrunch

ld hl,#_food2_food3_food4_0_bin
ld de,#0xb600
call _LZX0_decrunch

ld hl,#_food2_food3_food4_1_bin
ld de,#0xf600
call _LZX0_decrunch

ld hl,#_food5_food6_food7_0_bin
ld de,#0xbe00
call _LZX0_decrunch

ld hl,#_food5_food6_food7_1_bin
ld de,#0xfe00
jp _LZX0_decrunch

__endasm;
}

void prepareTile(){
__asm
prepareTile0:
ld bc,#_tblTileLocation_0
jr prepareTile

prepareTile1:
ld bc,#_tblTileLocation_1

prepareTile:

ld hl,(_tileID)
ld h,#0
add hl,hl
add hl,bc

ld a,(hl)
inc hl
ld h,(hl)
ld l,a

exx

PREPARE_TILE_SET_PAGE:
ld h,#0xFF

ld a,(_posY)
add a,a
ld l,a

ld bc,(_posX)
srl c

exx
__endasm;
}

void printTile_16x16_0()__naked{
__asm
call prepareTile0
ld bc,#0x1000

tileRow16_0:

ld a,c
add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

ld a,b

exx
ld b,a
ROW16_0:
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ldi
ldi
ldi
ldi
ldi
ldi
ldi
ldi
inc hl

exx
djnz ROW16_0
exx

ret
__endasm;
}

void printTile_16x16_1()__naked{
__asm
call prepareTile1
ld bc,#0x1000

tileRow16_1:

ld a,c
add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

ld a,b

exx
ld b,a
ROW16_1:
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ld a,(de)
and a,#ISOLATE_LEFT
or a,(hl)
ld (de),a

inc l
inc e

ldi
ldi
ldi
ldi
ldi
ldi
ldi

ld a,(de)
and a,#ISOLATE_RIGHT
or a,(hl)
ld (de),a

inc hl

exx
djnz ROW16_1
exx

ret
__endasm;
}

void printTile_15x16_0()__naked{
__asm
call prepareTile0
ld bc,#0x1000

tileRow15_0:

ld a,c
add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

ld a,b

exx
ld b,a
ROW15_0:
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ldi
ldi
ldi
ldi

set 6,h
inc l

ldi
ldi
ldi

ld a,(de)
and a,#ISOLATE_RIGHT
or a,(hl)
ld (de),a

res 6,h
inc hl

exx
djnz ROW15_0
exx

ret
__endasm;
}

void printTile_15x16_1()__naked{
__asm
call prepareTile1
ld bc,#0x1000

tileRow15_1:

ld a,c
add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

ld a,b

exx
ld b,a
ROW15_1:
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ld a,(de)
and a,#ISOLATE_LEFT
or a,(hl)
ld (de),a

inc l
inc e

ldi
ldi
ldi

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
res 6,h
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc e

ldi
ldi
ldi

set 6,h
inc hl

exx
djnz ROW15_1
exx

ret
__endasm;
}

void printTile_16x15_0()__naked{
__asm
call prepareTile0
ld bc,#0x0800
call tileRow16_0
ld bc,#0x0709
jp tileRow16_0
__endasm;
}

void printTile_16x15_1()__naked{
__asm
call prepareTile1
ld bc,#0x0800
call tileRow16_1
ld bc,#0x0709
jp tileRow16_1
__endasm;
}

void printTile_15x15_0()__naked{
__asm
call prepareTile0
ld bc,#0x0800
call tileRow15_0
ld bc,#0x0709
jp tileRow15_0
__endasm;
}

void printTile_15x15_1()__naked{
__asm
call prepareTile1
ld bc,#0x0800
call tileRow15_1
ld bc,#0x0709
jp tileRow15_1
__endasm;
}

void printTile_14x15_0()__naked{
__asm
call prepareTile0
ld bc,#0x0800
call tileRow14_0
ld bc,#0x0709

tileRow14_0:

ld a,c
add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

ld a,b

exx
ld b,a
ROW14_0:
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ldi
ldi

set 6,h
inc l

ldi
ldi
ldi

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
res 6,h
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc e

ldi

inc hl

exx
djnz ROW14_0
exx

ret
__endasm;
}

void printTile_14x15_1()__naked{
__asm
call prepareTile1
ld bc,#0x0800
call tileRow14_1
ld bc,#0x0709

tileRow14_1:

ld a,c
add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

ld a,b

exx
ld b,a
ROW14_1:
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ld a,(de)
and a,#ISOLATE_LEFT
or a,(hl)
ld (de),a

inc l
inc e

ldi

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
res 6,h
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc e

ldi
ldi
ldi

set 6,h
inc l

ldi

ld a,(de)
and a,#ISOLATE_RIGHT
or a,(hl)
ld (de),a

inc hl

exx
djnz ROW14_1
exx

ret
__endasm;
}

void printTile_15x14_0()__naked{
__asm
call prepareTile0
ld bc,#0x0400
call tileRow15_0
ld bc,#0x0709
call tileRow15_0
ld bc,#0x0309
jp tileRow15_0
__endasm;
}

void printTile_15x14_1()__naked{
__asm
call prepareTile1
ld bc,#0x0400
call tileRow15_1
ld bc,#0x0709
call tileRow15_1
ld bc,#0x0309
jp tileRow15_1
__endasm;
}

void printTile_14x14_0()__naked{
__asm
call prepareTile0
ld bc,#0x0400
call tileRow14_0
ld bc,#0x0709
call tileRow14_0
ld bc,#0x0309
jp tileRow14_0
__endasm;
}

void printTile_14x14_1()__naked{
__asm
call prepareTile1
ld bc,#0x0400
call tileRow14_1
ld bc,#0x0709
call tileRow14_1
ld bc,#0x0309
jp tileRow14_1
__endasm;
}

void printTile_13x14_0()__naked{
__asm
call prepareTile0
ld bc,#0x0400
call tileRow13_0
ld bc,#0x0709
call tileRow13_0
ld bc,#0x0309

tileRow13_0:

ld a,c
add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

ld a,b

exx
ld b,a
ROW13_0:
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ldi

set 6,h
inc l

ldi
ldi

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
res 6,h
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc e

ldi

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
set 6,h
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc e

ld a,(de)
and a,#ISOLATE_RIGHT
or a,(hl)
ld (de),a

res 6,h
inc hl

exx
djnz ROW13_0
exx

ret
__endasm;
}

void printTile_13x14_1()__naked{
__asm
call prepareTile1
ld bc,#0x0400
call tileRow13_1
ld bc,#0x0709
call tileRow13_1
ld bc,#0x0309

tileRow13_1:

ld a,c
add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

ld a,b

exx
ld b,a
ROW13_1:
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ld a,(de)
and a,#ISOLATE_LEFT
or a,(hl)
ld (de),a

inc l
inc e

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
res 6,h
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc e

ldi
ldi

set 6,h
inc l

ldi
ldi

res 6,h

ldi

set 6,h
inc hl

exx
djnz ROW13_1
exx

ret
__endasm;
}

void printTile_14x13_0()__naked{
__asm
call prepareTile0
ld bc,#0x0200
call tileRow14_0
ld bc,#0x0509
call tileRow14_0
ld bc,#0x0409
call tileRow14_0
ld bc,#0x0209
jp tileRow14_0
__endasm;
}

void printTile_14x13_1()__naked{
__asm
call prepareTile1
ld bc,#0x0200
call tileRow14_1
ld bc,#0x0509
call tileRow14_1
ld bc,#0x0409
call tileRow14_1
ld bc,#0x0209
jp tileRow14_1
__endasm;
}

void printTile_13x13_0()__naked{
__asm
call prepareTile0
ld bc,#0x0200
call tileRow13_0
ld bc,#0x0509
call tileRow13_0
ld bc,#0x0409
call tileRow13_0
ld bc,#0x0209
jp tileRow13_0
__endasm;
}

void printTile_13x13_1()__naked{
__asm
call prepareTile1
ld bc,#0x0200
call tileRow13_1
ld bc,#0x0509
call tileRow13_1
ld bc,#0x0409
call tileRow13_1
ld bc,#0x0209
jp tileRow13_1
__endasm;
}

void printTile_12x13_0()__naked{
__asm
call prepareTile0
ld bc,#0x0200
call tileRow12_0
ld bc,#0x0509
call tileRow12_0
ld bc,#0x0409
call tileRow12_0
ld bc,#0x0209

tileRow12_0:

ld a,c
add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

ld a,b

exx
ld b,a
ROW12_0:
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ldi

set 6,h
inc l

ldi

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
res 6,h
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc e

ldi

set 6,h
inc l

ldi

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
res 6,h
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc hl

exx
djnz ROW12_0
exx

ret
__endasm;
}

void printTile_12x13_1()__naked{
__asm
call prepareTile1
ld bc,#0x0200
call tileRow12_1
ld bc,#0x0509
call tileRow12_1
ld bc,#0x0409
call tileRow12_1
ld bc,#0x0209

tileRow12_1:

ld a,c
add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

ld a,b

exx
ld b,a
ROW12_1:
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ld a,(de)
and a,#ISOLATE_LEFT
or a,(hl)
ld (de),a

inc l
inc e

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
res 6,h
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc e

ldi

set 6,h
inc l

ldi

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
res 6,h
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc e

ldi

set 6,h
inc l

ld a,(de)
and a,#ISOLATE_RIGHT
or a,(hl)
ld (de),a

inc hl

exx
djnz ROW12_1
exx

ret
__endasm;
}

void printTile_13x12_0()__naked{
__asm
call prepareTile0
ld bc,#0x0200
call tileRow13_0
ld bc,#0x0309
call tileRow13_0
ld bc,#0x0309
call tileRow13_0
ld bc,#0x0309
call tileRow13_0
ld bc,#0x0109
jp tileRow13_0
__endasm;
}

void printTile_13x12_1()__naked{
__asm
call prepareTile1
ld bc,#0x0200
call tileRow13_1
ld bc,#0x0309
call tileRow13_1
ld bc,#0x0309
call tileRow13_1
ld bc,#0x0309
call tileRow13_1
ld bc,#0x0109
jp tileRow13_1
__endasm;
}

void printTile_12x12_0()__naked{
__asm
call prepareTile0
ld bc,#0x0200
call tileRow12_0
ld bc,#0x0309
call tileRow12_0
ld bc,#0x0309
call tileRow12_0
ld bc,#0x0309
call tileRow12_0
ld bc,#0x0109
jp tileRow12_0
__endasm;
}

void printTile_12x12_1()__naked{
__asm
call prepareTile1
ld bc,#0x0200
call tileRow12_1
ld bc,#0x0309
call tileRow12_1
ld bc,#0x0309
call tileRow12_1
ld bc,#0x0309
call tileRow12_1
ld bc,#0x0109
jp tileRow12_1
__endasm;
}

void printTile_11x12_0()__naked{
__asm
call prepareTile0
xor a
call tileRow11_0
call tileRow11_0
ld a,#9
call tileRow11_0
call tileRow11_0
call tileRow11_0
ld a,#9
call tileRow11_0
call tileRow11_0
call tileRow11_0
ld a,#9
call tileRow11_0
call tileRow11_0
call tileRow11_0
ld a,#9

tileRow11_0:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
set 6,h
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
res 6,h
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc e

ldi

set 6,h
inc l

ldi

res 6,h

ldi

set 6,h
inc l

ld a,(de)
and a,#ISOLATE_RIGHT
or a,(hl)
ld (de),a

res 6,h
inc hl

xor a

ret
__endasm;
}

void printTile_11x12_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
xor a
call tileRow11_1
call tileRow11_1
ld a,#9
call tileRow11_1
call tileRow11_1
call tileRow11_1
ld a,#9
call tileRow11_1
call tileRow11_1
call tileRow11_1
ld a,#9
call tileRow11_1
call tileRow11_1
call tileRow11_1
ld a,#9

tileRow11_1:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ld a,(de)
and a,c
or a,(hl)
ld (de),a

res 6,h
inc l
inc e

ldi
inc c

set 6,h
inc l

ldi
inc c

ld a,(hl)
and a,c
ld b,a
res 6,h
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
ld b,a
set 6,h
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
ld b,a
res 6,h
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

set 6,h
inc l
inc hl

xor a

ret
__endasm;
}

void printTile_12x11_0()__naked{
__asm
call prepareTile0
ld bc,#0x0100
call tileRow12_0
ld bc,#0x0209
call tileRow12_0
ld bc,#0x0309
call tileRow12_0
ld bc,#0x0209
call tileRow12_0
ld bc,#0x0209
call tileRow12_0
ld bc,#0x0109
jp tileRow12_0
__endasm;
}

void printTile_12x11_1()__naked{
__asm
call prepareTile1
ld bc,#0x0100
call tileRow12_1
ld bc,#0x0209
call tileRow12_1
ld bc,#0x0309
call tileRow12_1
ld bc,#0x0209
call tileRow12_1
ld bc,#0x0209
call tileRow12_1
ld bc,#0x0109
jp tileRow12_1
__endasm;
}

void printTile_11x11_0()__naked{
__asm
call prepareTile0
xor a
call tileRow11_0
ld a,#9
call tileRow11_0
call tileRow11_0
ld a,#9
call tileRow11_0
call tileRow11_0
call tileRow11_0
ld a,#9
call tileRow11_0
call tileRow11_0
ld a,#9
call tileRow11_0
call tileRow11_0
ld a,#9
jp tileRow11_0
__endasm;
}

void printTile_11x11_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
xor a
call tileRow11_1
ld a,#9
call tileRow11_1
call tileRow11_1
ld a,#9
call tileRow11_1
call tileRow11_1
call tileRow11_1
ld a,#9
call tileRow11_1
call tileRow11_1
ld a,#9
call tileRow11_1
call tileRow11_1
ld a,#9
jp tileRow11_1
__endasm;
}

void printTile_10x11_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
xor a
call tileRow10_0
ld a,#9
call tileRow10_0
call tileRow10_0
ld a,#9
call tileRow10_0
call tileRow10_0
call tileRow10_0
ld a,#9
call tileRow10_0
call tileRow10_0
ld a,#9
call tileRow10_0
call tileRow10_0
ld a,#9

tileRow10_0:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
set 6,h
inc l
ld a,(hl)
and a,c
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
res 6,h
ld a,(hl)
and a,c
or a,b
ld (de),a

set 6,h
inc l
inc l
inc e

ldi
inc c

res 6,h

ldi
inc c

set 6,h
inc l

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
res 6,h
ld a,(hl)
and a,c
or a,b
ld (de),a

inc l
inc hl

xor a

ret
__endasm;
}

void printTile_10x11_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
xor a
call tileRow10_1
ld a,#9
call tileRow10_1
call tileRow10_1
ld a,#9
call tileRow10_1
call tileRow10_1
call tileRow10_1
ld a,#9
call tileRow10_1
call tileRow10_1
ld a,#9
call tileRow10_1
call tileRow10_1
ld a,#9

tileRow10_1:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ld a,(de)
and a,c
or a,(hl)
ld (de),a

res 6,h
inc l
inc e

ldi
inc c

set 6,h
inc l

ld a,(hl)
and a,c
ld b,a
res 6,h
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
ld b,a
set 6,h
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
ld b,a
res 6,h
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

set 6,h
inc l
inc l
inc e

ld a,(de)
and a,#ISOLATE_RIGHT
or a,(hl)
ld (de),a

inc hl

xor a

ret
__endasm;
}

void printTile_11x10_0()__naked{
__asm
call prepareTile0
xor a
call tileRow11_0
ld a,#9
call tileRow11_0
call tileRow11_0
ld a,#9
call tileRow11_0
ld a,#9
call tileRow11_0
call tileRow11_0
ld a,#9
call tileRow11_0
call tileRow11_0
ld a,#9
call tileRow11_0
ld a,#9
jp tileRow11_0
__endasm;
}

void printTile_11x10_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
xor a
call tileRow11_1
ld a,#9
call tileRow11_1
call tileRow11_1
ld a,#9
call tileRow11_1
ld a,#9
call tileRow11_1
call tileRow11_1
ld a,#9
call tileRow11_1
call tileRow11_1
ld a,#9
call tileRow11_1
ld a,#9
jp tileRow11_1
__endasm;
}

void printTile_10x10_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
xor a
call tileRow10_0
ld a,#9
call tileRow10_0
call tileRow10_0
ld a,#9
call tileRow10_0
ld a,#9
call tileRow10_0
call tileRow10_0
ld a,#9
call tileRow10_0
call tileRow10_0
ld a,#9
call tileRow10_0
ld a,#9
jp tileRow10_0
__endasm;
}

void printTile_10x10_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
xor a
call tileRow10_1
ld a,#9
call tileRow10_1
call tileRow10_1
ld a,#9
call tileRow10_1
ld a,#9
call tileRow10_1
call tileRow10_1
ld a,#9
call tileRow10_1
call tileRow10_1
ld a,#9
call tileRow10_1
ld a,#9
jp tileRow10_1
__endasm;
}

void printTile_9x10_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
xor a
call tileRow9_0
ld a,#9
call tileRow9_0
call tileRow9_0
ld a,#9
call tileRow9_0
ld a,#9
call tileRow9_0
call tileRow9_0
ld a,#9
call tileRow9_0
call tileRow9_0
ld a,#9
call tileRow9_0
ld a,#9

tileRow9_0:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
inc l
ld a,(hl)
rrca
and a,c
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
inc l
ld a,(hl)
rrca
and a,c
or a,b
ld (de),a

inc e

ld a,(hl)
and a,c
rlca
ld b,a
inc l
ld a,(hl)
and a,c
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
rlca
ld b,a
inc l
ld a,(hl)
and a,c
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
rlca
ld b,a
ld a,(de)
and a,c
or a,b
ld (de),a

inc l
inc hl

xor a

ret
__endasm;
}

void printTile_9x10_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
xor a
call tileRow9_1
ld a,#9
call tileRow9_1
call tileRow9_1
ld a,#9
call tileRow9_1
ld a,#9
call tileRow9_1
call tileRow9_1
ld a,#9
call tileRow9_1
call tileRow9_1
ld a,#9
call tileRow9_1
ld a,#9

tileRow9_1:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ld a,(de)
and a,c
or a,(hl)
ld (de),a

inc l
inc e

ld a,(hl)
rlca
and a,c
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
rlca
and a,c
ld b,a
inc l
ld a,(hl)
and a,c
rrca
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
ld b,a
inc l
ld a,(hl)
and a,c
rrca
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
ld b,a
inc l
ld a,(hl)
and a,c
rrca
or a,b
ld (de),a

inc hl

xor a

ret
__endasm;
}

void printTile_10x9_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
xor a
call tileRow10_0
ld a,#9
call tileRow10_0
ld a,#9
call tileRow10_0
ld a,#9
call tileRow10_0
call tileRow10_0
ld a,#9
call tileRow10_0
ld a,#9
call tileRow10_0
ld a,#9
call tileRow10_0
ld a,#9
jp tileRow10_0
__endasm;
}

void printTile_10x9_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
xor a
call tileRow10_1
ld a,#9
call tileRow10_1
ld a,#9
call tileRow10_1
ld a,#9
call tileRow10_1
call tileRow10_1
ld a,#9
call tileRow10_1
ld a,#9
call tileRow10_1
ld a,#9
call tileRow10_1
ld a,#9
jp tileRow10_1
__endasm;
}

void printTile_9x9_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
xor a
call tileRow9_0
ld a,#9
call tileRow9_0
ld a,#9
call tileRow9_0
ld a,#9
call tileRow9_0
call tileRow9_0
ld a,#9
call tileRow9_0
ld a,#9
call tileRow9_0
ld a,#9
call tileRow9_0
ld a,#9
jp tileRow9_0
__endasm;
}

void printTile_9x9_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
xor a
call tileRow9_1
ld a,#9
call tileRow9_1
ld a,#9
call tileRow9_1
ld a,#9
call tileRow9_1
call tileRow9_1
ld a,#9
call tileRow9_1
ld a,#9
call tileRow9_1
ld a,#9
call tileRow9_1
ld a,#9
jp tileRow9_1
__endasm;
}

void printTile_8x9_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_LEFT
xor a
call tileRow8_0
ld a,#9
call tileRow8_0
ld a,#9
call tileRow8_0
ld a,#9
call tileRow8_0
call tileRow8_0
ld a,#9
call tileRow8_0
ld a,#9
call tileRow8_0
ld a,#9
call tileRow8_0
ld a,#9

tileRow8_0:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ld a,(hl)
and a,c
ld b,a
inc l
ld a,(hl)
and a,c
rrca
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
ld b,a
inc l
ld a,(hl)
and a,c
rrca
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
ld b,a
inc l
ld a,(hl)
and a,c
rrca
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
ld b,a
inc l
ld a,(hl)
and a,c
rrca
or a,b
ld (de),a

inc l
inc hl

xor a

ret
__endasm;
}

void printTile_8x9_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_RIGHT
xor a
call tileRow8_1
ld a,#9
call tileRow8_1
ld a,#9
call tileRow8_1
ld a,#9
call tileRow8_1
call tileRow8_1
ld a,#9
call tileRow8_1
ld a,#9
call tileRow8_1
ld a,#9
call tileRow8_1
ld a,#9

tileRow8_1:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ld a,(de)
and a,#ISOLATE_LEFT
or a,(hl)
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
rlca
ld b,a
inc l
ld a,(hl)
and a,c
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
rlca
ld b,a
inc l
ld a,(hl)
and a,c
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
rlca
ld b,a
inc l
ld a,(hl)
and a,c
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
rlca
ld b,a
ld a,(de)
and a,c
or a,b
ld (de),a

inc l
inc hl

xor a

ret
__endasm;
}

void printTile_9x8_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
xor a
call tileRow9_0
ld a,#9
call tileRow9_0
ld a,#9
call tileRow9_0
ld a,#9
call tileRow9_0
ld a,#9
call tileRow9_0
ld a,#9
call tileRow9_0
ld a,#9
call tileRow9_0
ld a,#9
jp tileRow9_0
__endasm;
}

void printTile_9x8_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
xor a
call tileRow9_1
ld a,#9
call tileRow9_1
ld a,#9
call tileRow9_1
ld a,#9
call tileRow9_1
ld a,#9
call tileRow9_1
ld a,#9
call tileRow9_1
ld a,#9
call tileRow9_1
ld a,#9
jp tileRow9_1
__endasm;
}

void printTile_8x8_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_LEFT
xor a
call tileRow8_0
ld a,#9
call tileRow8_0
ld a,#9
call tileRow8_0
ld a,#9
call tileRow8_0
ld a,#9
call tileRow8_0
ld a,#9
call tileRow8_0
ld a,#9
call tileRow8_0
ld a,#9
jp tileRow8_0
__endasm;
}

void printTile_8x8_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_RIGHT
xor a
call tileRow8_1
ld a,#9
call tileRow8_1
ld a,#9
call tileRow8_1
ld a,#9
call tileRow8_1
ld a,#9
call tileRow8_1
ld a,#9
call tileRow8_1
ld a,#9
call tileRow8_1
ld a,#9
jp tileRow8_1
__endasm;
}

void printTile_7x8_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
xor a
call tileRow7_0
ld a,#9
call tileRow7_0
ld a,#9
call tileRow7_0
ld a,#9
call tileRow7_0
ld a,#9
call tileRow7_0
ld a,#9
call tileRow7_0
ld a,#9
call tileRow7_0
ld a,#9

tileRow7_0:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ld a,(hl)
and a,c
rlca
ld b,a
inc l
ld a,(hl)
and a,c
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
rlca
ld b,a
inc l
ld a,(hl)
and a,c
or a,b
ld (de),a

inc l
inc l
inc e

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
inc l
ld a,(hl)
rrca
and a,c
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
ld a,(de)
and a,c
or a,b
ld (de),a

inc l
inc hl

ret
__endasm;
}

void printTile_7x8_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
xor a
call tileRow7_1
ld a,#9
call tileRow7_1
ld a,#9
call tileRow7_1
ld a,#9
call tileRow7_1
ld a,#9
call tileRow7_1
ld a,#9
call tileRow7_1
ld a,#9
call tileRow7_1
ld a,#9

tileRow7_1:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

inc l

ld a,(hl)
and a,c
rrca
ld b,a
ld a,(de)
and a,c
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
ld b,a
inc l
ld a,(hl)
and a,c
rrca
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
rlca
and a,c
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc hl

ret
__endasm;
}

void printTile_8x7_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_LEFT
ld a,#9
call tileRow8_0
ld a,#9
call tileRow8_0
ld a,#9
call tileRow8_0
ld a,#9
call tileRow8_0
ld a,#9*2
call tileRow8_0
ld a,#9
call tileRow8_0
ld a,#9
jp tileRow8_0
__endasm;
}

void printTile_8x7_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_RIGHT
ld a,#9
call tileRow8_1
ld a,#9
call tileRow8_1
ld a,#9
call tileRow8_1
ld a,#9
call tileRow8_1
ld a,#9*2
call tileRow8_1
ld a,#9
call tileRow8_1
ld a,#9
jp tileRow8_1
__endasm;
}

void printTile_7x7_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
ld a,#9
call tileRow7_0
ld a,#9
call tileRow7_0
ld a,#9
call tileRow7_0
ld a,#9
call tileRow7_0
ld a,#9*2
call tileRow7_0
ld a,#9
call tileRow7_0
ld a,#9
jp tileRow7_0
__endasm;
}

void printTile_7x7_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
ld a,#9
call tileRow7_1
ld a,#9
call tileRow7_1
ld a,#9
call tileRow7_1
ld a,#9
call tileRow7_1
ld a,#9*2
call tileRow7_1
ld a,#9
call tileRow7_1
ld a,#9
jp tileRow7_1
__endasm;
}

void printTile_6x7_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
ld a,#9
call tileRow6_0
ld a,#9
call tileRow6_0
ld a,#9
call tileRow6_0
ld a,#9
call tileRow6_0
ld a,#9*2
call tileRow6_0
ld a,#9
call tileRow6_0
ld a,#9

tileRow6_0:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ld a,(hl)
and a,c
rlca
ld b,a
inc l
ld a,(hl)
and a,c
or a,b
ld (de),a

inc l
inc l
inc e

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
inc l
ld a,(hl)
and a,c
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
rlca
ld b,a
inc l
inc l
ld a,(hl)
rrca
and a,c
or a,b
ld (de),a

inc l
inc hl

ret
__endasm;
}

void printTile_6x7_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
ld a,#9
call tileRow6_1
ld a,#9
call tileRow6_1
ld a,#9
call tileRow6_1
ld a,#9
call tileRow6_1
ld a,#9*2
call tileRow6_1
ld a,#9
call tileRow6_1
ld a,#9

tileRow6_1:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

inc l

ld a,(hl)
and a,c
rrca
ld b,a
ld a,(de)
and a,c
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc l
inc e

ld a,(hl)
and a,c
ld b,a
inc l
ld a,(hl)
and a,c
rrca
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
rlca
and a,c
ld b,a
ld a,(de)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc hl

ret
__endasm;
}

void printTile_7x6_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
ld a,#9
call tileRow7_0
ld a,#9
call tileRow7_0
ld a,#9*2
call tileRow7_0
ld a,#9*2
call tileRow7_0
ld a,#9
call tileRow7_0
ld a,#9*2
jp tileRow7_0
__endasm;
}

void printTile_7x6_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
ld a,#9
call tileRow7_1
ld a,#9
call tileRow7_1
ld a,#9*2
call tileRow7_1
ld a,#9*2
call tileRow7_1
ld a,#9
call tileRow7_1
ld a,#9*2
jp tileRow7_1
__endasm;
}

void printTile_6x6_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
ld a,#9
call tileRow6_0
ld a,#9
call tileRow6_0
ld a,#9*2
call tileRow6_0
ld a,#9*2
call tileRow6_0
ld a,#9
call tileRow6_0
ld a,#9*2
jp tileRow6_0
__endasm;
}

void printTile_6x6_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
ld a,#9
call tileRow6_1
ld a,#9
call tileRow6_1
ld a,#9*2
call tileRow6_1
ld a,#9*2
call tileRow6_1
ld a,#9
call tileRow6_1
ld a,#9*2
jp tileRow6_1
__endasm;
}

void printTile_5x6_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
ld a,#9
call tileRow5_0
ld a,#9
call tileRow5_0
ld a,#9*2
call tileRow5_0
ld a,#9*2
call tileRow5_0
ld a,#9
call tileRow5_0
ld a,#9*2

tileRow5_0:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ld a,(hl)
and a,c
rlca
ld b,a
inc l
inc l
ld a,(hl)
rrca
and a,c
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
and a,c
rlca
ld b,a
inc l
inc l
ld a,(hl)
and a,c
or a,b
ld (de),a

inc l
inc l
inc e

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
ld a,(de)
and a,c
or a,b
ld (de),a

inc l
inc hl

ret
__endasm;
}

void printTile_5x6_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
ld a,#9
call tileRow5_1
ld a,#9
call tileRow5_1
ld a,#9*2
call tileRow5_1
ld a,#9*2
call tileRow5_1
ld a,#9
call tileRow5_1
ld a,#9*2

tileRow5_1:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

inc l

ld a,(hl)
and a,c
rrca
ld b,a
ld a,(de)
and a,c
or a,b
ld (de),a

inc l
inc e

ld a,(hl)
rlca
and a,c
ld b,a
inc l
inc l
ld a,(hl)
and a,c
rrca
or a,b
ld (de),a

inc l
inc l
inc e

ld a,(hl)
and a,c
ld b,a
inc l
ld a,(hl)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc hl

ret
__endasm;
}

void printTile_6x5_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
ld a,#9
call tileRow6_0
ld a,#9*2
call tileRow6_0
ld a,#9*2
call tileRow6_0
ld a,#9*3
call tileRow6_0
ld a,#9*2
jp tileRow6_0
__endasm;
}

void printTile_6x5_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
ld a,#9
call tileRow6_1
ld a,#9*2
call tileRow6_1
ld a,#9*2
call tileRow6_1
ld a,#9*3
call tileRow6_1
ld a,#9*2
jp tileRow6_1
__endasm;
}

void printTile_5x5_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
ld a,#9
call tileRow5_0
ld a,#9*2
call tileRow5_0
ld a,#9*2
call tileRow5_0
ld a,#9*3
call tileRow5_0
ld a,#9*2
jp tileRow5_0
__endasm;
}

void printTile_5x5_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
ld a,#9
call tileRow5_1
ld a,#9*2
call tileRow5_1
ld a,#9*2
call tileRow5_1
ld a,#9*3
call tileRow5_1
ld a,#9*2
jp tileRow5_1
__endasm;
}

void printTile_4x5_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
ld a,#9
call tileRow4_0
ld a,#9*2
call tileRow4_0
ld a,#9*2
call tileRow4_0
ld a,#9*3
call tileRow4_0
ld a,#9*2

tileRow4_0:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

ld a,(hl)
and a,c
rlca
ld b,a
inc l
inc l
ld a,(hl)
and a,c
or a,b
ld (de),a

inc l
inc l
inc e

ld a,(hl)
and a,c
rlca
ld b,a
inc l
inc l
ld a,(hl)
and a,c
or a,b
ld (de),a

inc l
inc l
inc hl

ret
__endasm;
}

void printTile_4x5_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
ld a,#9
call tileRow4_1
ld a,#9*2
call tileRow4_1
ld a,#9*2
call tileRow4_1
ld a,#9*3
call tileRow4_1
ld a,#9*2

tileRow4_1:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

inc l

ld a,(hl)
and a,c
rrca
ld b,a
ld a,(de)
and a,c
or a,b
ld (de),a

inc l
inc l
inc e

ld a,(hl)
and a,c
ld b,a
inc l
inc l
ld a,(hl)
and a,c
rrca
or a,b
ld (de),a

inc l
inc l
inc e

ld a,(hl)
and a,c
ld b,a
ld a,(de)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc hl

ret
__endasm;
}

void printTile_5x4_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
ld a,#9
call tileRow5_0
ld a,#9*3
call tileRow5_0
ld a,#9*3
call tileRow5_0
ld a,#9*3
jp tileRow5_0
__endasm;
}

void printTile_5x4_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
ld a,#9
call tileRow5_1
ld a,#9*3
call tileRow5_1
ld a,#9*3
call tileRow5_1
ld a,#9*3
jp tileRow5_1
__endasm;
}

void printTile_4x4_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
ld a,#9
call tileRow4_0
ld a,#9*3
call tileRow4_0
ld a,#9*3
call tileRow4_0
ld a,#9*3
jp tileRow4_0
__endasm;
}

void printTile_4x4_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
ld a,#9
call tileRow4_1
ld a,#9*3
call tileRow4_1
ld a,#9*3
call tileRow4_1
ld a,#9*3
jp tileRow4_1
__endasm;
}

void printTile_3x4_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
ld a,#9
call tileRow3_0
ld a,#9*3
call tileRow3_0
ld a,#9*3
call tileRow3_0
ld a,#9*3

tileRow3_0:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

inc l

ld a,(hl)
and a,#ISOLATE_LEFT
ld b,a
inc l
inc l
ld a,(hl)
and a,c
or a,b
ld (de),a

inc l
inc l
inc l
inc e

ld a,(hl)
and a,c
rlca
ld b,a
ld a,(de)
and a,c
or a,b
ld (de),a

inc l
inc l
inc hl

ret
__endasm;
}

void printTile_3x4_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
ld a,#9
call tileRow3_1
ld a,#9*3
call tileRow3_1
ld a,#9*3
call tileRow3_1
ld a,#9*3

tileRow3_1:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

inc l

ld a,(hl)
and a,#ISOLATE_RIGHT
ld b,a
ld a,(de)
and a,c
or a,b
ld (de),a

inc l
inc l
inc l
inc e

ld a,(hl)
and a,c
ld b,a
inc l
inc l
inc l
ld a,(hl)
and a,c
rrca
or a,b
ld (de),a

inc l
inc hl

ret
__endasm;
}

void printTile_4x3_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
ld a,#9*2
call tileRow4_0
ld a,#9*4
call tileRow4_0
ld a,#9*5
jp tileRow4_0
__endasm;
}

void printTile_4x3_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
ld a,#9*2
call tileRow4_1
ld a,#9*4
call tileRow4_1
ld a,#9*5
jp tileRow4_1
__endasm;
}

void printTile_3x3_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
ld a,#9*2
call tileRow3_0
ld a,#9*4
call tileRow3_0
ld a,#9*5
jp tileRow3_0
__endasm;
}

void printTile_3x3_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
ld a,#9*2
call tileRow3_1
ld a,#9*4
call tileRow3_1
ld a,#9*5
jp tileRow3_1
__endasm;
}

void printTile_2x3_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
ld a,#9*2
call tileRow2_0
ld a,#9*4
call tileRow2_0
ld a,#9*5

tileRow2_0:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

inc l

ld a,(hl)
and a,c
rlca
ld b,a
ld a,#4
add a,l
ld l,a
ld a,(hl)
and a,c
or a,b
ld (de),a

inc l
inc l
inc l
inc hl

ret
__endasm;
}

void printTile_2x3_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
ld a,#9*2
call tileRow2_1
ld a,#9*4
call tileRow2_1
ld a,#9*5

tileRow2_1:

add a,l
ld l,a

ld a,h
adc a,#0
ld h,a

exx
ld a,(hl)
inc l
add a,c
ex af,af
ld a,(hl)
inc l
exx

ld d,a
ex af,af
ld e,a

inc l
inc l

ld a,(hl)
and a,c
rrca
ld b,a
ld a,(de)
and a,c
or a,b
ld (de),a

ld a,#4
add a,l
ld l,a
inc e

ld a,(hl)
and a,c
ld b,a
ld a,(de)
and a,#ISOLATE_RIGHT
or a,b
ld (de),a

inc l
inc l
inc hl

ret
__endasm;
}

void printTile_3x2_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
ld a,#9*3
call tileRow3_0
ld a,#9*7
jp tileRow3_0
__endasm;
}

void printTile_3x2_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
ld a,#9*3
call tileRow3_1
ld a,#9*7
jp tileRow3_1
__endasm;
}

void printTile_2x2_0()__naked{
__asm
call prepareTile0
ld c,#ISOLATE_RIGHT
ld a,#9*3
call tileRow2_0
ld a,#9*7
jp tileRow2_0
__endasm;
}

void printTile_2x2_1()__naked{
__asm
call prepareTile1
ld c,#ISOLATE_LEFT
ld a,#9*3
call tileRow2_1
ld a,#9*7
jp tileRow2_1
__endasm;
}

void sfxBlock(){
    if ((sfxFoodPlayed==0) && (sfxAngryFacePlayed==0) && (sfxUrchinPlayed==0) && (sfxEolPlayed==0)){
        silence();
        cpct_akp_SFXPlay(9,15,40,0,0,AY_CHANNEL_A);
        cpct_akp_SFXPlay(9,15,48,0,0,AY_CHANNEL_C);
    }
}

void sfxFood(u8 note){
    if ((sfxAngryFacePlayed==0) && (sfxUrchinPlayed==0)){
        silence();
        cpct_akp_SFXPlay(14,10,note+16,2,0,AY_CHANNEL_A);
        cpct_akp_SFXPlay(9,15,48,0,0,AY_CHANNEL_C);
        sfxFoodPlayed=1;
    }
}

void sfxDeath(u8 instrument){
    silence();

    if (instrument==10){
        cpct_akp_SFXPlay(instrument,10,56,0,0,AY_CHANNEL_A);
    }
    else{
        cpct_akp_SFXPlay(instrument,10,40,0,0,AY_CHANNEL_A);
    }

    cpct_akp_SFXPlay(instrument,15,48,0,0,AY_CHANNEL_C);
}

void collision8(){
    for (u8 i=0;i<5;i++){
        if (tblCollision[i].tileID!=255){
            if (tblCollision[i].tileID==TILE_URCHIN){
                sfxAngryFacePlayed=1;
            }
            else if ((tblCollision[i].tileID>=TILE_BIG_URCHIN_BEGIN) && (tblCollision[i].tileID<=TILE_BIG_URCHIN_END)){
                sfxUrchinPlayed=1;
            }
        }
    }

    if (sfxUrchinPlayed==1){
        for (u8 i=0;i<5;i++){
            if ((tblCollision[i].tileID>=TILE_BIG_URCHIN_BEGIN) && (tblCollision[i].tileID<=TILE_BIG_URCHIN_END)){
                tblCollision[i].tileID=255;
            }
        }

        foodDisplay=FOOD_DISPLAY;
        energy-=5*energyDecrement;

        if (energy>128){
            energy=0;
        }

        __asm
        ld hl,#_fonts_ouch2_bin
        call _printFood
        call _printEnergy
        __endasm;
    }
    else if (sfxAngryFacePlayed==1){
        for (u8 i=0;i<5;i++){
            if (tblCollision[i].tileID==TILE_URCHIN){
                tblCollision[i].tileID=255;
            }
        }

        foodDisplay=FOOD_DISPLAY;
        energy-=2*energyDecrement;

        if (energy>128){
            energy=0;
        }

        __asm
        ld hl,#_fonts_ouch1_bin
        call _printFood
        call _printEnergy
        __endasm;
    }
}

void collision15(){
    silence();
    cpct_akp_SFXPlay(16,10,50,3,0,AY_CHANNEL_A);
    cpct_akp_SFXPlay(16,15,60,3,0,AY_CHANNEL_C);
    sfxEolPlayed=1;
}

void collision16(){
    sfxFood(48);
    updateScore(1);

    __asm
    ld hl,#_fonts_apple_bin
    jp _printFood
    __endasm;
}

void collision17(){
    sfxFood(49);
    updateScore(2);

    __asm
    ld hl,#_fonts_banana_bin
    jp _printFood
    __endasm;
}

void collision18(){
    sfxFood(50);
    updateScore(3);

    __asm
    ld hl,#_fonts_egg_bin
    jp _printFood
    __endasm;
}

void collision19(){
    sfxFood(51);
    updateScore(4);

    __asm
    ld hl,#_fonts_cupcake_bin
    jp _printFood
    __endasm;
}

void collision20(){
    sfxFood(52);
    updateScore(5);

    __asm
    ld hl,#_fonts_milkshake_bin
    jp _printFood
    __endasm;
}

void collision21(){
    sfxFood(53);
    updateScore(6);

    __asm
    ld hl,#_fonts_chicken_bin
    jp _printFood
    __endasm;
}

void collision22(){
    sfxFood(54);
    updateScore(7);

    __asm
    ld hl,#_fonts_hamburger_bin
    jp _printFood
    __endasm;
}

void collision23(){
    sfxFood(55);
    updateScore(8);

    __asm
    ld hl,#_fonts_pizza_bin
    jp _printFood
    __endasm;
}

void printNumber(u8* vmem,u16 var,u8 digits){
    u8 reps;
    u8 digit;
    u16 tmp=var;
    u16 offsetChunky;

    if (var<10){
        reps=1;
    }
    else if (var<100){
        reps=2;
    }
    else if (var<1000){
        reps=3;
    }
    else if (var<10000){
        reps=4;
    }
    else{
        reps=5;
    }

    if ((gameState==STATE_MAIN_MENU) || (gameState==STATE_GAME_OVER)){
        for (u8 i=0;i!=digits;i++){
            if (i<reps){
                digit=tmp%10;
                tmp-=digit;
                tmp/=10;
            }
            else{
                digit=0;
            }

            if (gameState==STATE_MAIN_MENU){
                offsetChunky=64*17+(64-5*4)/2+4*4-i*4;
            }
            else{
                offsetChunky=64*12+(64-5*4)/2+4*4-i*4;
            }

            printChar(12,digit+48,offsetChunky);
        }
    }
    else{
        for (u8 i=0;i!=digits;i++){
            if (i<reps){
                digit=tmp%10;
                tmp-=digit;
                tmp/=10;
                pTile=asciiChar[digit];
            }
            else{
                pTile=asciiChar[0];
            }

            pmem=vmem+8-i*2;
            printTile();
        }
    }
}

// void printDiag(){
//     printNumber(CPCT_VMEM_START+64*20,player_z,5);
//     printNumber(SCREEN_BUFFER+64*20,player_z,5);
// }

void printScore(){
    printNumber(CPCT_VMEM_START+64*23+12,score,5);
    printNumber(SCREEN_BUFFER+64*23+12,score,5);
}

void updateScore(u8 scoreIncrease)__z88dk_fastcall{
    score+=scoreIncrease;
    energy+=scoreIncrease;
    energyDecrementDelay=ENERGY_DECREMENT_DELAY-currentLevel;
    foodDisplay=FOOD_DISPLAY;

    if (energy>100){
        energy=100;
    }

    printScore();
    printEnergy();

    if (score>highScore){
        highScore=score;
        printHighScore();
    }

    // toPrintScore=2;
}

void printHighScore(){
    printNumber(CPCT_VMEM_START+64*23+42,highScore,5);
    printNumber(SCREEN_BUFFER+64*23+42,highScore,5);
}

void printEnergy(){
    printNumber(CPCT_VMEM_START+64*23+25,energy,3);
    printNumber(SCREEN_BUFFER+64*23+25,energy,3);
}

void updateEnergy(){
    energyDecrementDelay--;

    if (energyDecrementDelay==0){
        energyDecrementDelay=ENERGY_DECREMENT_DELAY-currentLevel;
        energy-=energyDecrement;

        if (energy>128){
            energy=0;
        }

        printEnergy();
    }

    if (energy==0){
        gameState=STATE_DEAD;
    }
}

void updateFoodDisplay(){
    if (foodDisplay!=0){
        foodDisplay--;

        if (foodDisplay==0){
            __asm
            ld hl,#_fonts_blueBar_bin
            // call _printFood
            jp _printFood
            __endasm;
        }
    }
}

void printFood(){
    decrunchFont();
    cpct_drawSprite(spriteScratch,CPCT_VMEM_START+64*21+21,22,8);
    cpct_drawSprite(spriteScratch,SCREEN_BUFFER+64*21+21,22,8);
}

void print_mainMenu(){
    printString(1,"(I)",64*9+(64-14*4)/2,3*4);
    printString(7,"NSTRUCTIONS",64*9+(64-14*4)/2+3*4,11*4);
    printString(1,"(C)",64*11+(64-9*4)/2,3*4);
    printString(7,"REDITS",64*11+(64-9*4)/2+3*4,6*4);
    printString(5,"HIGH SCORE",64*15+(64-10*4)/2,10*4);
    printNumber(NULL,highScore,5);
    print_pressAnyKey();
}

void print_instructions(){
    printString(7,"EAT TO SURVIVE",64*0+(64-14*4)/2,14*4);
    printString(1,"JOYSTICK/ARROWS",64*4+(64-15*4)/2,15*4);
    printString(7,"to SWIM",64*6+(64-7*4)/2,7*4);
    printString(1,"FIRE/SPACE",64*10+(64-10*4)/2,10*4);
    printString(7,"to DIVE",64*12+(64-7*4)/2,7*4);
    printString(1,"(P)",64*16+(64-12*4)/2,3*4);
    printString(3," to PAUSE",64*16+(64-12*4)/2+3*4,9*4);
    printString(1,"(ESC)",64*18+(64-13*4)/2,5*4);
    printString(3," to QUIT",64*18+(64-13*4)/2+5*4,8*4);
    print_pressAnyKey();
}

void print_credits(){
    printString(7,"Ervin Pajor",64*0+(64-11*4)/2,11*4);
    printString(7,"2022",64*2+(64-4*4)/2,4*4);
    printString(14,"GOATFISH",64*5+(64-8*4)/2,8*4);
    printString(14,"devised by",64*7+(64-10*4)/2,10*4);
    printString(14,"Antonia",64*9+(64-7*4)/2,7*4);
    printString(14,"Music by",64*12+(64-8*4)/2,8*4);
    printString(14,"Antonia",64*14+(64-7*4)/2,7*4);
    printString(9,"Urchins by",64*17+(64-10*4)/2,10*4);
    printString(9,"Zoe",64*19+(64-3*4)/2,3*4);
    print_pressAnyKey();
}

void print_gameOver(){
    printString(5,"GAME OVER",64*0+(64-9*4)/2,9*4);
    printString(1,"SCORE",64*10+(64-5*4)/2,5*4);
    printNumber(NULL,score,5);
    print_pressAnyKey();
}

void print_nextLevel(){
    printString(1,"KEEP GOING!",64*10+(64-11*4)/2,11*4);
    printString(1,"GOAT HARDER!",64*12+(64-12*4)/2,12*4);
}

void print_congratulations(){
    printString(1,"CONGRATULATIONS!",64*8+(64-16*4)/2,16*4);
    printString(1,"GOATFISH",64*12+(64-8*4)/2,8*4);
    printString(1,"IS FULL!",64*14+(64-8*4)/2,8*4);
    printChar(7,224,64*18+(64-1*4)/2);
}

void print_dead(){
    printString(1,"GOATFISH",64*9+(64-8*4)/2,8*4);
    printString(1,"HAS RUN OUT",64*11+(64-11*4)/2,11*4);
    printString(1,"OF ENERGY",64*13+(64-9*4)/2,9*4);
    printChar(5,225,64*17+(64-1*4)/2);
}

void print_pressAnyKey(){
    printString(7,"PRESS ANY KEY",64*22+(64-13*4)/2,13*4);
}

void printChunkyText(u16 offset,u8 bytes){
    u8* vmemTo=CPCT_VMEM_START+offset;
    u8* vmemFrom=SCREEN_BUFFER;

    for (u8 i=0;i<8;i++){
        cpct_memcpy(vmemTo,vmemFrom,bytes);
        vmemTo+=0x0800;
        cpct_memcpy(vmemTo,vmemFrom,bytes);
        vmemFrom+=0x0800;

        if (i==3){
            vmemTo-=0x3800-64;
        }
        else{
            vmemTo+=0x0800;
        }
    }
}

void printChar(u8 colour,u8 ascii,u16 offsetChunky){
    cpct_setDrawCharM0(colour,0);
    cpct_drawCharM0(SCREEN_BUFFER,ascii);
    printChunkyText(offsetChunky,1*4);
}

void printString(u8 colour,u8* s,u16 offsetChunky,u8 bytesChunky){
    cpct_setDrawCharM0(colour,0);
    cpct_drawStringM0(s,SCREEN_BUFFER);
    printChunkyText(offsetChunky,bytesChunky);
}

void printWordSprite(u8* tmem,u8 width,u8 height){
    cpct_drawSprite(spriteScratch,tmem,width,height);
    cpct_drawSprite(spriteScratch,tmem-0x4000,width,height);
}

void urchin_small_up(){
    // if (ptrUrchinInstance->position<=7){
    //     ptrUrchinInstance->direction=URCHIN_8_DOWN;
    //     ptrUrchinInstance->position+=8;
    // }
    // else{
    //     ptrUrchinInstance->position-=8;
    // }

__asm

;src/main.c:6303: if (ptrUrchinInstance->position<=7){
    ld  hl, (_ptrUrchinInstance)
    ld  a, #0x07
    sub a,(hl)
    jr  C,00102$
;src/main.c:6304: ptrUrchinInstance->direction=URCHIN_8_DOWN;
    inc hl
    ld  (hl), #0x52
;src/main.c:6303: if (ptrUrchinInstance->position<=7){
    dec hl
;src/main.c:6305: ptrUrchinInstance->position+=8;
    ld  a, (hl)
    add a, #0x08
    ld  (hl), a
    ret
00102$:
;src/main.c:6308: ptrUrchinInstance->position-=8;
    ld a,(hl)
    add a, #0xf8
    ld  (hl), a

__endasm;
}

void urchin_small_down(){
    // if (ptrUrchinInstance->position>=32){
    //     ptrUrchinInstance->direction=URCHIN_8_UP;
    //     ptrUrchinInstance->position-=8;
    // }
    // else{
    //     ptrUrchinInstance->position+=8;
    // }

__asm

;src/main.c:6313: if (ptrUrchinInstance->position>=32){
    ld  hl, (_ptrUrchinInstance)
    ld  a, (hl)
    cp  a, #0x20
    jr  C,00102$
;src/main.c:6314: ptrUrchinInstance->direction=URCHIN_8_UP;
    inc hl
    ld  (hl), #0x51
;src/main.c:6313: if (ptrUrchinInstance->position>=32){
    dec hl
;src/main.c:6315: ptrUrchinInstance->position-=8;
    ld  a, (hl)
    add a, #0xf8
    ld  (hl), a
    ret
00102$:
;src/main.c:6318: ptrUrchinInstance->position+=8;
    add a, #0x08
    ld  (hl), a

__endasm;
}

void urchin_small_left(){
    // if (ptrUrchinInstance->position%8==0){
    //     ptrUrchinInstance->direction=URCHIN_8_RIGHT;
    //     ptrUrchinInstance->position++;
    // }
    // else{
    //     ptrUrchinInstance->position--;
    // }

__asm

;src/main.c:6323: if (ptrUrchinInstance->position%8==0){
    ld  hl, (_ptrUrchinInstance)
    ld  a, (hl)
    and a, #0x07
    jr  NZ,00102$
;src/main.c:6324: ptrUrchinInstance->direction=URCHIN_8_RIGHT;
    inc hl
    ld  (hl), #0x54
;src/main.c:6325: ptrUrchinInstance->position++;
    dec hl
    inc (hl)
    ret
00102$:
;src/main.c:6328: ptrUrchinInstance->position--;
    dec (hl)

__endasm;
}

void urchin_small_right(){
    // if (ptrUrchinInstance->position%8==7){
    //     ptrUrchinInstance->direction=URCHIN_8_LEFT;
    //     ptrUrchinInstance->position--;
    // }
    // else{
    //     ptrUrchinInstance->position++;
    // }

__asm

;src/main.c:6333: if (ptrUrchinInstance->position%8==7){
    ld  hl, (_ptrUrchinInstance)
    ld a,(hl)
    and a, #0x07
    sub a, #0x07
    jr  NZ,00102$
;src/main.c:6334: ptrUrchinInstance->direction=URCHIN_8_LEFT;
    inc hl
    ld  (hl), #0x53
;src/main.c:6335: ptrUrchinInstance->position--;
    dec hl
    dec (hl)
    ret
00102$:
;src/main.c:6338: ptrUrchinInstance->position++;
    inc (hl)

__endasm;
}

void urchin_big_up(){
    // if (ptrUrchinInstance->position<=7){
    //     ptrUrchinInstance->direction=URCHIN_9_DOWN;
    //     ptrUrchinInstance->position+=8;
    // }
    // else{
    //     ptrUrchinInstance->position-=8;
    // }

__asm

;src/main.c:6343: if (ptrUrchinInstance->position<=7){
    ld  hl, (_ptrUrchinInstance)
    ld  a, #0x07
    sub a, (hl)
    jr  C,00102$
;src/main.c:6344: ptrUrchinInstance->direction=URCHIN_9_DOWN;
    inc hl
    ld  (hl), #0x5c
;src/main.c:6343: if (ptrUrchinInstance->position<=7){
    dec hl
;src/main.c:6345: ptrUrchinInstance->position+=8;
    ld  a, (hl)
    add a, #0x08
    ld  (hl), a
    ret
00102$:
;src/main.c:6348: ptrUrchinInstance->position-=8;
    ld  a, (hl)
    add a, #0xf8
    ld  (hl), a

__endasm;
}

void urchin_big_down(){
    // if (ptrUrchinInstance->position>=24){
    //     ptrUrchinInstance->direction=URCHIN_9_UP;
    //     ptrUrchinInstance->position-=8;
    // }
    // else{
    //     ptrUrchinInstance->position+=8;
    // }

__asm

;src/main.c:6353: if (ptrUrchinInstance->position>=24){
    ld  hl, (_ptrUrchinInstance)
    ld  a, (hl)
    cp  a, #0x18
    jr  C,00102$
;src/main.c:6354: ptrUrchinInstance->direction=URCHIN_9_UP;
    inc hl
    ld  (hl), #0x5b
;src/main.c:6353: if (ptrUrchinInstance->position>=24){
    dec hl
;src/main.c:6355: ptrUrchinInstance->position-=8;
    ld  a, (hl)
    add a, #0xf8
    ld  (hl), a
    ret
00102$:
;src/main.c:6358: ptrUrchinInstance->position+=8;
    add a, #0x08
    ld  (hl), a

__endasm;
}

void urchin_big_left(){
    // if (ptrUrchinInstance->position%8==0){
    //     ptrUrchinInstance->direction=URCHIN_9_RIGHT;
    //     ptrUrchinInstance->position++;
    // }
    // else{
    //     ptrUrchinInstance->position--;
    // }

__asm

;src/main.c:6363: if (ptrUrchinInstance->position%8==0){
    ld  hl, (_ptrUrchinInstance)
    ld  a, (hl)
    and a, #0x07
    jr  NZ,00102$
;src/main.c:6364: ptrUrchinInstance->direction=URCHIN_9_RIGHT;
    inc hl
    ld  (hl), #0x5e
;src/main.c:6365: ptrUrchinInstance->position++;
    dec hl
    inc (hl)
    ret
00102$:
;src/main.c:6368: ptrUrchinInstance->position--;
    dec (hl)

__endasm;
}

void urchin_big_right(){
    // if (ptrUrchinInstance->position%8==6){
    //     ptrUrchinInstance->direction=URCHIN_9_LEFT;
    //     ptrUrchinInstance->position--;
    // }
    // else{
    //     ptrUrchinInstance->position++;
    // }

__asm

;src/main.c:6373: if (ptrUrchinInstance->position%8==6){
    ld  hl, (_ptrUrchinInstance)
    ld  a, (hl)
    and a, #0x07
    sub a, #0x06
    jr  NZ,00102$
;src/main.c:6374: ptrUrchinInstance->direction=URCHIN_9_LEFT;
    inc hl
    ld  (hl), #0x5d
;src/main.c:6375: ptrUrchinInstance->position--;
    dec hl
    dec (hl)
    ret
00102$:
;src/main.c:6378: ptrUrchinInstance->position++;
    inc (hl)

__endasm;
}

void prepareUrchins(){
    // cpct_memset_f64(tblUrchinX,#0x0000,40*12);

    // zoneIndex=player_z/1024-1;

    // if (zoneIndex==0){
    //     zoneFrom=0;
    // }
    // else{
    //     zoneFrom=zoneIndex-1;
    // }

    // zoneTo=zoneIndex+1;

    // for (zoneIndex=zoneFrom;zoneIndex<=zoneTo;zoneIndex++){
    //     if (urchins[zoneIndex]!=0){
    //         ptrFloorUrchin=urchins[zoneIndex];

    //         while (ptrFloorUrchin->tblUrchin!=0){
    //             ptrUrchinInstance=ptrFloorUrchin->urchinInstance;

    //             while (ptrUrchinInstance->position!=0xFF){
    //                 urchinDelay=ptrUrchinInstance->speed_delay & 0b00001111;

    //                 if (ptrUrchinInstance->direction<URCHIN_9_STATIONARY){
    //                     if (urchinDelay==0){
    //                         urchinDelay=ptrUrchinInstance->speed_delay>>4;

    //                         if (ptrUrchinInstance->direction>URCHIN_8_STATIONARY){
    //                             tblUrchin_small[ptrUrchinInstance->direction-URCHIN_8_STATIONARY-1]();
    //                         }
    //                     }

    //                     ptrFloorUrchin->tblUrchin[ptrUrchinInstance->position]=0b01001000;
    //                 }                        
    //                 else{
    //                     if (urchinDelay==0){
    //                         urchinDelay=ptrUrchinInstance->speed_delay>>4;

    //                         if (ptrUrchinInstance->direction>URCHIN_9_STATIONARY){
    //                             tblUrchin_big[ptrUrchinInstance->direction-URCHIN_9_STATIONARY-1]();
    //                         }
    //                     }

    //                     ptrFloorUrchin->tblUrchin[ptrUrchinInstance->position]=0b01001001;
    //                     ptrFloorUrchin->tblUrchin[ptrUrchinInstance->position+1]=0b01001010;
    //                     ptrFloorUrchin->tblUrchin[ptrUrchinInstance->position+8]=0b01001011;
    //                     ptrFloorUrchin->tblUrchin[ptrUrchinInstance->position+8+1]=0b01001100;
    //                 }

    //                 if (urchinDelay!=0){
    //                     urchinDelay--;
    //                 }

    //                 ptrUrchinInstance->speed_delay=(ptrUrchinInstance->speed_delay & 0b11110000) | urchinDelay;
    //                 ptrUrchinInstance++;   
    //             }

    //             ptrFloorUrchin++;
    //         }
    //     }
    // }

__asm

;src/main.c:6572: cpct_memset_f64(tblUrchinX,#0x0000,40*12);
    ld hl,#480
    push    hl
    ld  hl, #0x0000
    push    hl
    ld  hl, #_tblUrchinX
    push    hl
    call    _cpct_memset_f8
;src/main.c:6574: zoneIndex=player_z/1024-1;
    ld  a,(#_player_z + 1)
    rrca
    rrca
    and a, #0x3f
    ld  hl, #_zoneIndex
    add a, #0xff
    ld  (hl), a
;src/main.c:6576: if (zoneIndex==0){
    ld  a,(#_zoneIndex + 0)
    or  a, a
    jr  NZ,00102$
;src/main.c:6577: zoneFrom=0;
    ld  hl,#_zoneFrom + 0
    ld  (hl), #0x00
    jr  00103$
00102$:
;src/main.c:6580: zoneFrom=zoneIndex-1;
    ld  hl, #_zoneFrom
    ld  a,(#_zoneIndex + 0)
    add a, #0xff
    ld  (hl), a
00103$:
;src/main.c:6583: zoneTo=zoneIndex+1;
    ld  hl, #_zoneTo
    ld  a,(#_zoneIndex + 0)
    inc a
    ld  (hl), a
;src/main.c:6585: for (zoneIndex=zoneFrom;zoneIndex<=zoneTo;zoneIndex++){
    ld  a,(#_zoneFrom + 0)
    ld  (#_zoneIndex + 0),a
00127$:
    ld  a, (#_zoneTo)
    ld hl,#_zoneIndex
    sub a,(hl)
    ret c
;src/main.c:6586: if (urchins[zoneIndex]!=0){
    ld  l,(hl)
    ld  h, #0x00
    add hl, hl
    ld  bc, #_urchins
    add hl, bc
    ld  c, (hl)
    inc hl
    ld  b, (hl)
    ld  a, b
    or  a,c
    jp  Z, 00128$
;src/main.c:6587: ptrFloorUrchin=urchins[zoneIndex];
    ld  (_ptrFloorUrchin), bc
;src/main.c:6589: while (ptrFloorUrchin->tblUrchin!=0){
00120$:
    ld  hl, (_ptrFloorUrchin)
    ld  c, (hl)
    inc hl
    ld  b, (hl)
    ld  a, b
    or  a,c
    jp  Z, 00128$
;src/main.c:6590: ptrUrchinInstance=ptrFloorUrchin->urchinInstance;
    inc hl
    ld a,(hl)
    inc hl
    ld h,(hl)
    ld l,a
    ld (_ptrUrchinInstance),hl
;src/main.c:6592: while (ptrUrchinInstance->position!=0xFF){
00117$:
    ld  hl, (_ptrUrchinInstance)
    ld  c, (hl)
    inc c
    jp  Z,00119$
;src/main.c:6593: urchinDelay=ptrUrchinInstance->speed_delay & 0b00001111;
    inc hl
    inc hl
    ld  b, (hl)
    ld  a, b
    and a, #0x0f
    ld d,a
;src/main.c:6595: if (ptrUrchinInstance->direction<URCHIN_9_STATIONARY){
    dec hl
    ld  c, (hl)
;src/main.c:6597: urchinDelay=ptrUrchinInstance->speed_delay>>4;
    ld  a, b
    rlca
    rlca
    rlca
    rlca
    and a, #0x0f
    ld  e, a
;src/main.c:6595: if (ptrUrchinInstance->direction<URCHIN_9_STATIONARY){
    ld  a, c
    sub a, #0x5a
    jr  NC,00113$
;src/main.c:6596: if (urchinDelay==0){
    ld a,d
    or  a, a
    jr  NZ,00107$
;src/main.c:6597: urchinDelay=ptrUrchinInstance->speed_delay>>4;
    ld d,e
;src/main.c:6599: if (ptrUrchinInstance->direction>URCHIN_8_STATIONARY){
    ld  a, #0x50
    sub a, c
    jr  NC,00107$
;src/main.c:6600: tblUrchin_small[ptrUrchinInstance->direction-URCHIN_8_STATIONARY-1]();
    ld  a, c
    add a, #0xaf
    ld  l, a
    ld  h, #0x00
    add hl, hl
    ld  bc, #_tblUrchin_small
    add hl, bc
    ld  c, (hl)
    inc hl
    ld  h, (hl)
    ld  l, c
    call    ___sdcc_call_hl
00107$:
;src/main.c:6604: ptrFloorUrchin->tblUrchin[ptrUrchinInstance->position]=0b01001000;
    ld  hl, (_ptrFloorUrchin)
    ld  c, (hl)
    inc hl
    ld  b, (hl)
    ld  hl, (_ptrUrchinInstance)
    ld  l, (hl)
    ld  h,#0x00
    add hl, bc
    ld  (hl), #0x48
    jr  00114$
00113$:
;src/main.c:6607: if (urchinDelay==0){
    ld a,d
    or  a, a
    jr  NZ,00111$
;src/main.c:6608: urchinDelay=ptrUrchinInstance->speed_delay>>4;
    ld d,e
;src/main.c:6610: if (ptrUrchinInstance->direction>URCHIN_9_STATIONARY){
    ld  a, #0x5a
    sub a, c
    jr  NC,00111$
;src/main.c:6611: tblUrchin_big[ptrUrchinInstance->direction-URCHIN_9_STATIONARY-1]();
    ld  a, c
    add a, #0xa5
    ld  l, a
    ld  h, #0x00
    add hl, hl
    ld  bc, #_tblUrchin_big
    add hl, bc
    ld  c, (hl)
    inc hl
    ld  h, (hl)
    ld  l, c
    call    ___sdcc_call_hl
00111$:
;src/main.c:6617: ptrFloorUrchin->tblUrchin[ptrUrchinInstance->position]=0b01001001;
    ld  hl, (_ptrFloorUrchin)
    ld  c, (hl)
    inc hl
    ld  b, (hl)
    ld  hl, (_ptrUrchinInstance)
    ld  l, (hl)
    ld  h,#0x00
    add hl, bc
    ld  (hl), #0x49
;src/main.c:6618: ptrFloorUrchin->tblUrchin[ptrUrchinInstance->position+1]=0b01001010;
    inc hl
    ld  (hl), #0x4a
;src/main.c:6619: ptrFloorUrchin->tblUrchin[ptrUrchinInstance->position+8]=0b01001011;
    ld bc,#7
    add hl,bc
    ld  (hl), #0x4b
;src/main.c:6620: ptrFloorUrchin->tblUrchin[ptrUrchinInstance->position+8+1]=0b01001100;
    inc hl
    ld  (hl), #0x4c
00114$:
;src/main.c:6623: if (urchinDelay!=0){
    ld a,d
    or  a, a
    jr  Z,00116$
;src/main.c:6624: urchinDelay--;
    dec d
00116$:
;src/main.c:6592: while (ptrUrchinInstance->position!=0xFF){
    ld  bc, (_ptrUrchinInstance)
;src/main.c:6593: urchinDelay=ptrUrchinInstance->speed_delay & 0b00001111;
    inc bc
    inc bc
;src/main.c:6627: ptrUrchinInstance->speed_delay=(ptrUrchinInstance->speed_delay & 0b11110000) | urchinDelay;
    ld  a, (bc)
    and a, #0xf0
    or a,d
    ld  (bc), a
;src/main.c:6628: ptrUrchinInstance++;   
    ld  hl, #_ptrUrchinInstance
    ld  a, (hl)
    add a, #0x03
    ld  (hl), a
    inc hl
    ld  a, (hl)
    adc a, #0x00
    ld  (hl), a
    jp  00117$
00119$:
;src/main.c:6631: ptrFloorUrchin++;
    ld  hl, #_ptrFloorUrchin
    ld  a, (hl)
    add a, #0x04
    ld  (hl), a
    inc hl
    ld  a, (hl)
    adc a, #0x00
    ld  (hl), a
    jp  00120$
00128$:
;src/main.c:6585: for (zoneIndex=zoneFrom;zoneIndex<=zoneTo;zoneIndex++){
    ld  hl, #_zoneIndex+0
    inc (hl)
    jp  00127$

__endasm;
}

void printTile(){
__asm

di

ld (SAVE_SP_PRINT_TILE+1),sp
ld sp,(_pTile)

ld hl,(_pmem)
pop de
ld (hl),e
inc l
ld (hl),d

set 3,h
pop de
ld (hl),d
dec l
ld (hl),e

pop bc

set 4,h
pop de
ld (hl),e
inc l
ld (hl),d

res 3,h
ld (hl),b
dec l
ld (hl),c

res 4,h
set 5,h
pop de
ld (hl),e
inc l
ld (hl),d

set 3,h
pop de
ld (hl),d
dec l
ld (hl),e

pop bc

set 4,h
pop de
ld (hl),e
inc l
ld (hl),d

res 3,h
ld (hl),b
dec l
ld (hl),c

SAVE_SP_PRINT_TILE:
ld sp,#0xffff

ei

__endasm;
}

void deexo() __naked{
    __asm

ld iy,#_spriteScratch+11
    ld a,(hl)
    inc hl
    ld b,#52
    push de
    cp a

exo_initbits:
    ld c,#16
    jr nz,exo_get4bits
    .db 0xdd
    ld l,c
    ld de,#1

exo_get4bits:
    call exo_getbit
    rl c
    jr nc,exo_get4bits
    inc c
    push hl
    ld hl,#1
    ld 41(iy),c

exo_setbit:
    dec c
    jr nz,#exo_setbit-1
    ld -11(iy),e
    ld 93(iy),d
    add hl,de
    ex de,hl
    inc iy
    pop hl
    .db 0xdd
    dec l
    djnz exo_initbits
    pop de
    jr exo_mainloop

exo_literalrun:
    ld e,c

exo_getbits:
    dec b
    ret z

exo_getbits1:
    call exo_getbit
    rl e
    rl d
    jr nc,exo_getbits
    ld b,d
    ld c,e
    pop de

exo_literalcopy:
    ldir

exo_mainloop:
    inc c
    call exo_getbit
    jr c,exo_literalcopy
    ld c, #239

exo_getindex:
    call exo_getbit
    inc c
    jr nc,exo_getindex
    ret z
    push de
    ld d,b
    jp p,exo_literalrun
    ld iy,#_spriteScratch-229
    call exo_getpair
    push de
    rlc d
    jr nz,exo_dontgo
    dec e
    ld bc,#512+32
    jr z,exo_goforit
    dec e

exo_dontgo:
    ld bc,#1024+16
    jr z,exo_goforit
    ld de,#0
    ld c,d

exo_goforit:
    call exo_getbits1
    ld iy,#_spriteScratch+27
    add iy,de
    call exo_getpair
    pop bc
    ex (sp),hl
    push hl
    sbc hl,de
    pop de
    ldir
    pop hl
    jr exo_mainloop

exo_getpair:
    add iy,bc
    ld e,d
    ld b,41(iy)
    call exo_getbits
    ex de,hl
    ld c,-11(iy)
    ld b,93(iy)
    add hl,bc
    ex de,hl
    ret

exo_getbit:
    srl a
    ret nz
    ld a,(hl)
    inc hl
    rra
    ret

    __endasm;
}
